@@ -2,7 +2,7 @@
     This class handles the bookmarks.
                              -------------------
     begin                : Fr Jan 9 2004
-    copyright            : (C) 2004-2016 by Alexander Reinholdt
+    copyright            : (C) 2004-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -48,7 +48,6 @@
 #include <QXmlStreamReader>
 #include <QPointer>
 #include <QMutableListIterator>
-#include <QStandardPaths>
 
 // KDE includes
 #define TRANSLATION_DOMAIN "smb4k-core"
@@ -69,7 +68,7 @@ Smb4KBookmarkHandler::Smb4KBookmarkHandler(QObject *parent)
   d->editor = 0;
   
   // First we need the directory.
-  QString path = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+  QString path = dataLocation();
   
   QDir dir;
   
@@ -82,11 +81,7 @@ Smb4KBookmarkHandler::Smb4KBookmarkHandler(QObject *parent)
     // Do nothing
   }
 
-  readBookmarks(&d->bookmarks, &d->groups, false);
-  
-  // Connections
-  connect(Smb4KProfileManager::self(), SIGNAL(activeProfileChanged(QString)), 
-          this, SLOT(slotActiveProfileChanged(QString)));
+  readBookmarks();
 }
 
 
@@ -94,7 +89,7 @@ Smb4KBookmarkHandler::~Smb4KBookmarkHandler()
 {
   while (!d->bookmarks.isEmpty())
   {
-    delete d->bookmarks.takeFirst();
+    d->bookmarks.takeFirst().clear();
   }
   
   delete d->editor;
@@ -107,11 +102,11 @@ Smb4KBookmarkHandler *Smb4KBookmarkHandler::self()
 }
 
 
-void Smb4KBookmarkHandler::addBookmark(Smb4KShare *share, QWidget *parent)
+void Smb4KBookmarkHandler::addBookmark(const SharePtr &share, QWidget *parent)
 {
   if (share)
   {
-    QList<Smb4KShare *> shares;
+    QList<SharePtr> shares;
     shares << share;
     addBookmarks(shares, parent);
   }
@@ -122,14 +117,18 @@ void Smb4KBookmarkHandler::addBookmark(Smb4KShare *share, QWidget *parent)
 }
 
 
-void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KShare *> &list, QWidget *parent)
+void Smb4KBookmarkHandler::addBookmarks(const QList<SharePtr> &list, QWidget *parent)
 {
-  // Prepare the list of bookmarks and show the save dialog.
-  QList<Smb4KBookmark *> new_bookmarks;
+  //
+  // Prepare the list of bookmarks that should be added
+  //
+  QList<BookmarkPtr> newBookmarks;
   
-  for (Smb4KShare *share : list)
+  for (const SharePtr &share : list)
   {
-    // Check if the share is a printer
+    //
+    // Printer shares cannot be bookmarked
+    //
     if (share->isPrinter())
     {
       Smb4KNotification::cannotBookmarkPrinter(share);
@@ -139,12 +138,12 @@ void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KShare *> &list, QWidget
     {
       // Do nothing
     }
-
+    
+    //
     // Process homes shares
+    //
     if (share->isHomesShare())
     {
-      // If the user provides a valid user name we set it and continue.
-      // Otherwise the share will be skipped.
       if (!Smb4KHomesSharesHandler::self()->specifyUser(share, true, parent))
       {
         continue;
@@ -154,22 +153,20 @@ void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KShare *> &list, QWidget
         // Do nothing
       }
     }
-
-    Smb4KBookmark *known_bookmark = 0;
-    
-    if (!share->isHomesShare())
-    {
-      known_bookmark = findBookmarkByUNC(share->unc());
-    }
     else
     {
-      known_bookmark = findBookmarkByUNC(share->homeUNC());
+      // Do nothing
     }
-
-    // Skip bookmarks already present
-    if (known_bookmark)
+    
+    //
+    // Check if the share has already been bookmarked and skip it if it
+    // already exists
+    //
+    BookmarkPtr knownBookmark = findBookmarkByUNC(share->isHomesShare() ? share->homeUNC() : share->unc());
+    
+    if (knownBookmark)
     {
-      Smb4KNotification::bookmarkExists(known_bookmark);
+      Smb4KNotification::bookmarkExists(knownBookmark.data());
       continue;
     }
     else
@@ -177,15 +174,18 @@ void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KShare *> &list, QWidget
       // Do nothing
     }
     
-    Smb4KBookmark *bookmark = new Smb4KBookmark(share);
+    BookmarkPtr bookmark = BookmarkPtr(new Smb4KBookmark(share.data()));
     bookmark->setProfile(Smb4KProfileManager::self()->activeProfile());
-    new_bookmarks << bookmark;
+    newBookmarks << bookmark;
   }
   
-  if (!new_bookmarks.isEmpty())
+  //
+  // Show the bookmark dialog, if necessary
+  //
+  if (!newBookmarks.isEmpty())
   {
-    QPointer<Smb4KBookmarkDialog> dlg = new Smb4KBookmarkDialog(new_bookmarks, groupsList(), parent);
-
+    QPointer<Smb4KBookmarkDialog> dlg = new Smb4KBookmarkDialog(newBookmarks, groupsList(), parent);
+    
     if (dlg->exec() == QDialog::Accepted)
     {
       // The bookmark dialog uses an internal list of bookmarks, 
@@ -205,84 +205,101 @@ void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KShare *> &list, QWidget
     // Do nothing
   }
   
-  // Clear the list of bookmarks.
-  while (!new_bookmarks.isEmpty())
+  //
+  // Clear the temporary list of bookmarks
+  //
+  while (!newBookmarks.isEmpty())
   {
-    delete new_bookmarks.takeFirst();
+    newBookmarks.takeFirst().clear();
   }
 }
 
 
-void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KBookmark*>& list, bool replace)
+void Smb4KBookmarkHandler::addBookmarks(const QList<BookmarkPtr> &list, bool replace)
 {
-  // Clear the internal lists if desired.
+  //
+  // Process the incoming list.
+  // In case the internal list should be replaced, clear the internal 
+  // list first.
+  // 
   if (replace)
   {
-    while (!d->bookmarks.isEmpty())
-    {
-      delete d->bookmarks.takeFirst();
-    }
+    QMutableListIterator<BookmarkPtr> it(d->bookmarks);
     
-    d->groups.clear();
+    while (it.hasNext())
+    {
+      BookmarkPtr bookmark = it.next();
+      
+      if (Smb4KSettings::useProfiles() && bookmark->profile() != Smb4KProfileManager::self()->activeProfile())
+      {
+        continue;
+      }
+      else
+      {
+        // Do nothing
+      }
+      
+      it.remove();
+    }
   }
   else
   {
     // Do nothing
   }
   
-  // Append the new bookmarks to the internal list and check
-  // the label simultaneously.
-  for (Smb4KBookmark *bookmark : list)
+  //
+  // Copy all bookmarks that are not in the list
+  // 
+  for (const BookmarkPtr &bookmark : list)
   {
+    //
+    // Check if the bookmark label is already in use
+    // 
     if (!bookmark->label().isEmpty() && findBookmarkByLabel(bookmark->label()))
     {
-      Smb4KNotification::bookmarkLabelInUse(bookmark);
-      Smb4KBookmark *new_bookmark = new Smb4KBookmark(*bookmark);
-      new_bookmark->setLabel(QString("%1 (1)").arg(bookmark->label()));
-      d->bookmarks << new_bookmark;
-    }
-    else
-    {
-      d->bookmarks << new Smb4KBookmark(*bookmark);
-    }
-  }
-      
-  // Append new groups to the internal list.
-  for (Smb4KBookmark *bookmark : list)
-  {
-    if (!d->groups.contains(bookmark->groupName()))
-    {
-      d->groups << bookmark->groupName();
+      Smb4KNotification::bookmarkLabelInUse(bookmark.data());
+      bookmark->setLabel(QString("%1 (1)").arg(bookmark->label()));
     }
     else
     {
       // Do nothing
     }
-  }
       
-  d->groups.sort();
-        
-  // Save the bookmarks list.
-  writeBookmarkList(d->bookmarks);  
+    //
+    // Check if we have to add the bookmark
+    // 
+    BookmarkPtr existingBookmark = findBookmarkByUNC(bookmark->unc());
+      
+    if (!existingBookmark)
+    {
+      d->bookmarks << bookmark;
+    }
+    else
+    {
+      // We do not to update the bookmark, because we are
+      // operating on a shared pointer.
+    }
+  }
+  
+  //
+  // Save the bookmark list and emit the updated() signal
+  //
+  writeBookmarkList();
   emit updated();
 }
 
 
-void Smb4KBookmarkHandler::removeBookmark(Smb4KBookmark* bookmark)
+void Smb4KBookmarkHandler::removeBookmark(const BookmarkPtr &bookmark)
 {
   if (bookmark)
   {
-    // Update the bookmarks
-    update();
-    
-    // NOTE: Do not convert to C++11 range-based for loop, since this loop
-    // uses QList::takeAt().
     for (int i = 0; i < d->bookmarks.size(); ++i)
     {
-      if (QString::compare(bookmark->unc(), d->bookmarks.at(i)->unc(), Qt::CaseInsensitive) == 0 &&
+      if ((!Smb4KSettings::useProfiles() || Smb4KSettings::activeProfile() == d->bookmarks.at(i)->profile()) &&
+          QString::compare(bookmark->unc(), d->bookmarks.at(i)->unc(), Qt::CaseInsensitive) == 0 &&
           QString::compare(bookmark->groupName(), d->bookmarks.at(i)->groupName(), Qt::CaseInsensitive) == 0)
       {
-        delete d->bookmarks.takeAt(i);
+        d->bookmarks.takeAt(i).clear();
         break;
       }
       else
@@ -291,25 +308,8 @@ void Smb4KBookmarkHandler::removeBookmark(Smb4KBookmark* bookmark)
       }
     }
     
-    // Update the groups
-    d->groups.clear();
-    
-    for (Smb4KBookmark *b : d->bookmarks)
-    {
-      if (!d->groups.contains(b->groupName()))
-      {
-        d->groups << b->groupName();
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-    
-    d->groups.sort();
-    
     // Write the list to the bookmarks file.
-    writeBookmarkList(d->bookmarks);
+    writeBookmarkList();
     emit updated();
   }
   else
@@ -321,15 +321,14 @@ void Smb4KBookmarkHandler::removeBookmark(Smb4KBookmark* bookmark)
 
 void Smb4KBookmarkHandler::removeGroup(const QString& name)
 {
-  update();
-  
-  QMutableListIterator<Smb4KBookmark *> it(d->bookmarks);
+  QMutableListIterator<BookmarkPtr> it(d->bookmarks);
   
   while (it.hasNext())
   {
-    Smb4KBookmark *b = it.next();
+    const BookmarkPtr &b = it.next();
     
-    if (QString::compare(b->groupName(), name, Qt::CaseInsensitive) == 0)
+    if ((!Smb4KSettings::useProfiles() || Smb4KSettings::activeProfile() == b->profile()) ||
+        QString::compare(b->groupName(), name, Qt::CaseInsensitive) == 0)
     {
       it.remove();
     }
@@ -339,69 +338,17 @@ void Smb4KBookmarkHandler::removeGroup(const QString& name)
     }
   }
   
-  // Update the groups list
-  d->groups.clear();
-  
-  for (Smb4KBookmark *bookmark : d->bookmarks)
-  {
-    if (!d->groups.contains(bookmark->groupName(), Qt::CaseInsensitive))
-    {
-      d->groups << bookmark->groupName();
-    }
-    else
-    {
-      // Do nothing
-    }
-  }
-  
-  d->groups.sort();
-  
   // Write the list to the bookmarks file.
-  writeBookmarkList(d->bookmarks);
+  writeBookmarkList();
   emit updated();
 }
 
 
-void Smb4KBookmarkHandler::writeBookmarkList(const QList<Smb4KBookmark *> &list, bool listOnly)
+void Smb4KBookmarkHandler::writeBookmarkList()
 {
-  QList<Smb4KBookmark *> allBookmarks;
-  QStringList allGroups; // Can be ignored.
-  
-  if (!listOnly)
-  {  
-    // First read all entries. Then remove all, that belong to
-    // the currently active profile.
-    readBookmarks(&allBookmarks, &allGroups, true);
-    
-    QMutableListIterator<Smb4KBookmark *> it(allBookmarks);
-    
-    while (it.hasNext())
-    {
-      Smb4KBookmark *bookmark = it.next();
-      
-      if (QString::compare(bookmark->profile(), Smb4KProfileManager::self()->activeProfile()) == 0)
-      {
-        it.remove();
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  for (Smb4KBookmark *bookmark : list)
-  {
-    allBookmarks << new Smb4KBookmark(*bookmark);
-  }
-  
-  QFile xmlFile(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation)+QDir::separator()+"bookmarks.xml");
+  QFile xmlFile(dataLocation()+QDir::separator()+"bookmarks.xml");
 
-  if (!allBookmarks.isEmpty())
+  if (!d->bookmarks.isEmpty())
   {
     if (xmlFile.open(QIODevice::WriteOnly|QIODevice::Text))
     {
@@ -411,7 +358,7 @@ void Smb4KBookmarkHandler::writeBookmarkList(const QList<Smb4KBookmark *> &list,
       xmlWriter.writeStartElement("bookmarks");
       xmlWriter.writeAttribute("version", "1.1");
 
-      for (Smb4KBookmark *bookmark : allBookmarks)
+      for (const BookmarkPtr &bookmark : d->bookmarks)
       {
         if (!bookmark->url().isValid())
         {
@@ -450,20 +397,13 @@ void Smb4KBookmarkHandler::writeBookmarkList(const QList<Smb4KBookmark *> &list,
   {
     xmlFile.remove();
   }
-  
-  while (!allBookmarks.isEmpty())
-  {
-    delete allBookmarks.takeFirst();
-  }
-  
-  allGroups.clear();
 }
 
 
-void Smb4KBookmarkHandler::readBookmarks(QList<Smb4KBookmark *> *bookmarks, QStringList *groups, bool allBookmarks)
+void Smb4KBookmarkHandler::readBookmarks()
 {
   // Locate the XML file.
-  QFile xmlFile(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation)+QDir::separator()+"bookmarks.xml");
+  QFile xmlFile(dataLocation()+QDir::separator()+"bookmarks.xml");
 
   if (xmlFile.open(QIODevice::ReadOnly | QIODevice::Text))
   {
@@ -487,71 +427,54 @@ void Smb4KBookmarkHandler::readBookmarks(QList<Smb4KBookmark *> *bookmarks, QStr
           {
             QString profile = xmlReader.attributes().value("profile").toString();
             
-            if (allBookmarks || QString::compare(Smb4KProfileManager::self()->activeProfile(), profile, Qt::CaseSensitive) == 0)
-            {
-              Smb4KBookmark *bookmark = new Smb4KBookmark();
-              bookmark->setProfile(profile);
-              bookmark->setGroupName(xmlReader.attributes().value("group").toString());
+            BookmarkPtr bookmark = BookmarkPtr(new Smb4KBookmark());
+            bookmark->setProfile(profile);
+            bookmark->setGroupName(xmlReader.attributes().value("group").toString());
               
-              while (!(xmlReader.isEndElement() && xmlReader.name() == "bookmark"))
+            while (!(xmlReader.isEndElement() && xmlReader.name() == "bookmark"))
+            {
+              xmlReader.readNext();
+
+              if (xmlReader.isStartElement())
               {
-                xmlReader.readNext();
-
-                if (xmlReader.isStartElement())
+                if (xmlReader.name() == "workgroup")
                 {
-                  if (xmlReader.name() == "workgroup")
-                  {
-                    bookmark->setWorkgroupName(xmlReader.readElementText());
-                  }
-                  else if (xmlReader.name() == "unc")
-                  {
-                    bookmark->setURL(xmlReader.readElementText());
-                  }
-                  else if (xmlReader.name() == "login")
-                  {
-                    bookmark->setLogin(xmlReader.readElementText());
-                  }
-                  else if (xmlReader.name() == "ip")
-                  {
-                    bookmark->setHostIP(xmlReader.readElementText());
-                  }
-                  else if (xmlReader.name() == "type")
-                  {
-                    bookmark->setTypeString(xmlReader.readElementText());
-                  }
-                  else if (xmlReader.name() == "label")
-                  {
-                    bookmark->setLabel(xmlReader.readElementText());
-                  }
-                  else
-                  {
-                    // Do nothing
-                  }
-
-                  continue;
+                  bookmark->setWorkgroupName(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "unc")
+                {
+                  bookmark->setURL(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "login")
+                {
+                  bookmark->setLogin(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "ip")
+                {
+                  bookmark->setHostIP(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "type")
+                {
+                  bookmark->setTypeString(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "label")
+                {
+                  bookmark->setLabel(xmlReader.readElementText());
                 }
                 else
                 {
-                  continue;
+                  // Do nothing
                 }
-              }
-              
-              *bookmarks << bookmark;
-              
-              if (!groups->contains(bookmark->groupName()))
-              {
-                *groups << bookmark->groupName();
+
+                continue;
               }
               else
               {
-                // Do nothing
-              }             
+                continue;
+              }
+            }
               
-            }
-            else
-            {
-              continue;
-            }
+            d->bookmarks << bookmark;
           }
           else
           {
@@ -592,15 +515,15 @@ void Smb4KBookmarkHandler::readBookmarks(QList<Smb4KBookmark *> *bookmarks, QStr
 }
 
 
-Smb4KBookmark *Smb4KBookmarkHandler::findBookmarkByUNC(const QString &unc)
+BookmarkPtr Smb4KBookmarkHandler::findBookmarkByUNC(const QString &unc)
 {
   // Update the bookmarks:
   update();
 
   // Find the bookmark:
-  Smb4KBookmark *bookmark = 0;
+  BookmarkPtr bookmark;
 
-  for (Smb4KBookmark *b : d->bookmarks)
+  for (const BookmarkPtr &b : bookmarksList())
   {
     if (QString::compare(b->unc().toUpper(), unc.toUpper()) == 0)
     {
@@ -617,15 +540,15 @@ Smb4KBookmark *Smb4KBookmarkHandler::findBookmarkByUNC(const QString &unc)
 }
 
 
-Smb4KBookmark *Smb4KBookmarkHandler::findBookmarkByLabel(const QString &label)
+BookmarkPtr Smb4KBookmarkHandler::findBookmarkByLabel(const QString &label)
 {
   // Update the bookmarks:
   update();
 
   // Find the bookmark:
-  Smb4KBookmark *bookmark = 0;
+  BookmarkPtr bookmark;
 
-  for (Smb4KBookmark *b : d->bookmarks)
+  for (const BookmarkPtr &b : bookmarksList())
   {
     if (QString::compare(b->label().toUpper(), label.toUpper()) == 0)
     {
@@ -642,25 +565,49 @@ Smb4KBookmark *Smb4KBookmarkHandler::findBookmarkByLabel(const QString &label)
 }
 
 
-QList<Smb4KBookmark *> Smb4KBookmarkHandler::bookmarksList() const
+QList<BookmarkPtr> Smb4KBookmarkHandler::bookmarksList() const
 {
   // Update the bookmarks:
   update();
 
+  // Get this list of the bookmarks
+  if (Smb4KSettings::useProfiles())
+  {
+    QList<BookmarkPtr> bookmarks;
+    
+    for (const BookmarkPtr &b : d->bookmarks)
+    {
+      if (b->profile() == Smb4KSettings::activeProfile())
+      {
+        bookmarks << b;
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    
+    return bookmarks;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
   // Return the list of bookmarks:
   return d->bookmarks;
 }
 
 
-QList<Smb4KBookmark *> Smb4KBookmarkHandler::bookmarksList(const QString &group) const
+QList<BookmarkPtr> Smb4KBookmarkHandler::bookmarksList(const QString &group) const
 {
   // Update bookmarks
   update();
 
   // Get the list of bookmarks organized in the given group
-  QList<Smb4KBookmark *> bookmarks;
+  QList<BookmarkPtr> bookmarks;
 
-  for (Smb4KBookmark *bookmark : d->bookmarks)
+  for (const BookmarkPtr &bookmark : bookmarksList())
   {
     if (QString::compare(group, bookmark->groupName(), Qt::CaseInsensitive) == 0)
     {
@@ -678,7 +625,21 @@ QList<Smb4KBookmark *> Smb4KBookmarkHandler::bookmarksList(const QString &group)
 
 QStringList Smb4KBookmarkHandler::groupsList() const
 {
-  return d->groups;
+  QStringList groups;
+  
+  for (const BookmarkPtr &b : bookmarksList())
+  {
+    if (!groups.contains(b->groupName()))
+    {
+      groups << b->groupName();
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  return groups;
 }
 
 
@@ -695,15 +656,12 @@ void Smb4KBookmarkHandler::editBookmarks(QWidget *parent)
   
   if (d->editor->exec() == QDialog::Accepted)
   {
-    // Now replace the current list with the new one that is
-    // passed by the editor. For this set the 'replace' argument
-    // for addBookmarks() to TRUE.
-    QList<Smb4KBookmark *> bookmarks = d->editor->editedBookmarks();
+    QList<BookmarkPtr> bookmarks = d->editor->editedBookmarks();
     addBookmarks(bookmarks, true);
   }
   else
   {
-    // Do nothing
+    resetBookmarks();
   }
 
   delete d->editor;
@@ -711,17 +669,22 @@ void Smb4KBookmarkHandler::editBookmarks(QWidget *parent)
 }
 
 
+void Smb4KBookmarkHandler::resetBookmarks()
+{
+  readBookmarks();
+}
+
+
 void Smb4KBookmarkHandler::update() const
 {
   // Get new IP addresses.
-  for (Smb4KBookmark *bookmark : d->bookmarks)
+  for (const BookmarkPtr &bookmark : d->bookmarks)
   {
-    Smb4KHost *host = findHost(bookmark->hostName(), bookmark->workgroupName());
+    HostPtr host = findHost(bookmark->hostName(), bookmark->workgroupName());
     
     if (host)
     {
-      if (!host->ip().trimmed().isEmpty() &&
-          QString::compare(bookmark->hostIP(), host->ip()) != 0)
+      if (host->hasIP() && bookmark->hostIP() != host->ip())
       {
         bookmark->setHostIP(host->ip());
       }
@@ -740,14 +703,8 @@ void Smb4KBookmarkHandler::update() const
 
 void Smb4KBookmarkHandler::migrateProfile(const QString& from, const QString& to)
 {
-  QList<Smb4KBookmark *> allBookmarks;
-  QStringList allGroups; // Can be ignored.
- 
-  // Read all entries for later conversion.
-  readBookmarks(&allBookmarks, &allGroups, true);
-  
   // Replace the old profile name with the new one.
-  for (Smb4KBookmark *bookmark : allBookmarks)
+  for (const BookmarkPtr &bookmark : d->bookmarks)
   {
     if (QString::compare(bookmark->profile(), from, Qt::CaseSensitive) == 0)
     {
@@ -760,34 +717,17 @@ void Smb4KBookmarkHandler::migrateProfile(const QString& from, const QString& to
   }
   
   // Write the new list to the file.
-  writeBookmarkList(allBookmarks, true);
-  
-  // Profile settings changed, so invoke the slot.
-  slotActiveProfileChanged(Smb4KProfileManager::self()->activeProfile());
-  
-  // Clear the temporary lists of bookmarks and groups.
-  while (!allBookmarks.isEmpty())
-  {
-    delete allBookmarks.takeFirst();
-  }
-  
-  allGroups.clear();
+  writeBookmarkList();
 }
 
 
 void Smb4KBookmarkHandler::removeProfile(const QString& name)
 {
-  QList<Smb4KBookmark *> allBookmarks;
-  QStringList allGroups; // Can be ignored.
- 
-  // Read all entries for later removal.
-  readBookmarks(&allBookmarks, &allGroups, true);
-  
-  QMutableListIterator<Smb4KBookmark *> it(allBookmarks);
+  QMutableListIterator<BookmarkPtr> it(d->bookmarks);
   
   while (it.hasNext())
   {
-    Smb4KBookmark *bookmark = it.next();
+    const BookmarkPtr &bookmark = it.next();
     
     if (QString::compare(bookmark->profile(), name, Qt::CaseSensitive) == 0)
     {
@@ -800,32 +740,6 @@ void Smb4KBookmarkHandler::removeProfile(const QString& name)
   }
   
   // Write the new list to the file.
-  writeBookmarkList(allBookmarks, true);
-  
-  // Profile settings changed, so invoke the slot.
-  slotActiveProfileChanged(Smb4KProfileManager::self()->activeProfile());
-  
-  // Clear the temporary lists of bookmarks and groups.
-  while (!allBookmarks.isEmpty())
-  {
-    delete allBookmarks.takeFirst();
-  }
-  
-  allGroups.clear();
-}
-
-
-void Smb4KBookmarkHandler::slotActiveProfileChanged(const QString &/*activeProfile*/)
-{
-  // Clear the list of bookmarks and the list of groups.
-  while (!d->bookmarks.isEmpty())
-  {
-    delete d->bookmarks.takeFirst();
-  }
-  
-  d->groups.clear();
-  
-  // Reload the bookmarks and groups.
-  readBookmarks(&d->bookmarks, &d->groups, false);
+  writeBookmarkList();
 }
 
