@@ -2,7 +2,7 @@
     Private classes for the bookmark handler
                              -------------------
     begin                : Sun Mar 20 2011
-    copyright            : (C) 2011-2016 by Alexander Reinholdt
+    copyright            : (C) 2011-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -28,7 +28,7 @@
 
 // application specific includes
 #include "smb4kbookmarkhandler.h"
-#include "smb4kbookmarkobject.h"
+#include "smb4kglobal.h"
 
 // Qt includes
 #include <QString>
@@ -38,6 +38,7 @@
 #include <QListWidget>
 #include <QAction>
 #include <QPushButton>
+#include <QPointer>
 
 // KDE includes
 #include <KCompletion/KLineEdit>
@@ -53,13 +54,13 @@ class Q_DECL_EXPORT Smb4KBookmarkDialog : public QDialog
     /**
      * The constructor
      *
-     * @param list            The list of bookmarks that are to be saved
+     * @param bookmarks       The list of bookmarks that are to be saved
      * 
      * @param groups          The list of available bookmark groups
      *
      * @param parent          The parent widget
      */
-    Smb4KBookmarkDialog(const QList<Smb4KBookmark *> &bookmarks,
+    Smb4KBookmarkDialog(const QList<BookmarkPtr> &bookmarks,
                         const QStringList &groups,
                         QWidget *parent);
 
@@ -74,7 +75,7 @@ class Q_DECL_EXPORT Smb4KBookmarkDialog : public QDialog
     * 
     * @returns the list of bookmarks.
     */
-   const QList<Smb4KBookmark *> &bookmarks();
+   const QList<BookmarkPtr> &bookmarks();
 
   protected Q_SLOTS:
     /**
@@ -111,12 +112,12 @@ class Q_DECL_EXPORT Smb4KBookmarkDialog : public QDialog
     /**
      * Load the list of bookmarks and the one of the groups
      */
-    void loadLists(const QList<Smb4KBookmark *> &bookmarks, const QStringList &groups);
+    void loadLists(const QList<BookmarkPtr> &bookmarks, const QStringList &groups);
 
     /**
      * Finds the bookmark in the list
      */
-    Smb4KBookmark *findBookmark(const QUrl &url);
+    BookmarkPtr findBookmark(const QUrl &url);
     
     /**
      * Ok push button
@@ -131,7 +132,7 @@ class Q_DECL_EXPORT Smb4KBookmarkDialog : public QDialog
     /**
      * The list of bookmarks
      */
-    QList<Smb4KBookmark *> m_bookmarks;
+    QList<BookmarkPtr> m_bookmarks;
 
     /**
      * The list of groups
@@ -172,8 +173,7 @@ class Q_DECL_EXPORT Smb4KBookmarkEditor : public QDialog
      *
      * @param parent      The parent of this dialog.
      */
-    explicit Smb4KBookmarkEditor(const QList<Smb4KBookmark *> &bookmarks,
-                                 QWidget *parent = 0);
+    explicit Smb4KBookmarkEditor(const QList<BookmarkPtr> &bookmarks, QWidget *parent = 0);
 
     /**
      * The destructor.
@@ -187,14 +187,13 @@ class Q_DECL_EXPORT Smb4KBookmarkEditor : public QDialog
      *
      * @returns the list of bookmarks
      */
-    QList<Smb4KBookmark *> editedBookmarks() const;
+    QList<BookmarkPtr> editedBookmarks() const;
 
   protected:
     /**
      * Reimplemented from QObject
      */
-    bool eventFilter(QObject *obj,
-                     QEvent *e);
+    bool eventFilter(QObject *obj, QEvent *e);
 
   protected Q_SLOTS:
     /**
@@ -263,12 +262,12 @@ class Q_DECL_EXPORT Smb4KBookmarkEditor : public QDialog
     /**
      * Load bookmarks
      */
-    void loadBookmarks(const QList<Smb4KBookmark *> &bookmarks);
+    void loadBookmarks();
 
     /**
      * Finds the bookmark in the list
      */
-    Smb4KBookmark *findBookmark(const QUrl &url);
+    BookmarkPtr findBookmark(const QUrl &url);
     
     /**
      * Ok push button
@@ -284,7 +283,7 @@ class Q_DECL_EXPORT Smb4KBookmarkEditor : public QDialog
     /**
      * List of the bookmarks that are being processed
      */
-    QList<Smb4KBookmark *> m_bookmarks;
+    QList<BookmarkPtr> m_bookmarks;
 
     /**
      * Tree widget
@@ -346,9 +345,8 @@ class Q_DECL_EXPORT Smb4KBookmarkEditor : public QDialog
 class Smb4KBookmarkHandlerPrivate
 {
   public:
-    Smb4KBookmarkEditor *editor;
-    QList<Smb4KBookmark *> bookmarks;
-    QStringList groups;
+    QPointer<Smb4KBookmarkEditor> editor;
+    QList<BookmarkPtr> bookmarks;
 };
 
 
