@@ -2,7 +2,7 @@
     This is the global namespace for Smb4K.
                              -------------------
     begin                : Sa Apr 2 2005
-    copyright            : (C) 2005-2016 by Alexander Reinholdt
+    copyright            : (C) 2005-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -52,11 +52,13 @@ Q_GLOBAL_STATIC(Smb4KGlobalPrivate, p);
 QMutex mutex(QMutex::Recursive /* needed to avoid dead-locks */);
 
 
-const QEvent::Type Smb4KGlobal::Smb4KEvent::LoadSettings =        (QEvent::Type)QEvent::registerEventType();
-const QEvent::Type Smb4KGlobal::Smb4KEvent::SetFocus =            (QEvent::Type)QEvent::registerEventType();
-const QEvent::Type Smb4KGlobal::Smb4KEvent::ScanNetwork =         (QEvent::Type)QEvent::registerEventType();
-const QEvent::Type Smb4KGlobal::Smb4KEvent::AddBookmark =         (QEvent::Type)QEvent::registerEventType();
-const QEvent::Type Smb4KGlobal::Smb4KEvent::MountOrUnmountShare = (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::LoadSettings =          (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::SetFocus =              (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::ScanNetwork =           (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::AddBookmark =           (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::MountOrUnmountShare =   (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::EnableBookmarkAction =  (QEvent::Type)QEvent::registerEventType();
+const QEvent::Type Smb4KGlobal::Smb4KEvent::DisableBookmarkAction = (QEvent::Type)QEvent::registerEventType();
 
 
 Smb4KGlobal::Smb4KEvent::Smb4KEvent(QEvent::Type type): QEvent(type)
@@ -128,19 +130,19 @@ bool Smb4KGlobal::coreIsInitialized()
 }
 
 
-const QList<Smb4KWorkgroup *> &Smb4KGlobal::workgroupsList()
+const QList<WorkgroupPtr> &Smb4KGlobal::workgroupsList()
 {
   return p->workgroupsList;
 }
 
 
-Smb4KWorkgroup *Smb4KGlobal::findWorkgroup(const QString &name)
+WorkgroupPtr Smb4KGlobal::findWorkgroup(const QString &name)
 {
-  Smb4KWorkgroup *workgroup = 0;
+  WorkgroupPtr workgroup;
 
   mutex.lock();
   
-  for (Smb4KWorkgroup *w : p->workgroupsList)
+  for (const WorkgroupPtr &w : p->workgroupsList)
   {
     if (QString::compare(w->workgroupName(), name, Qt::CaseInsensitive) == 0)
     {
@@ -159,74 +161,122 @@ Smb4KWorkgroup *Smb4KGlobal::findWorkgroup(const QString &name)
 }
 
 
-bool Smb4KGlobal::addWorkgroup(Smb4KWorkgroup *workgroup)
+bool Smb4KGlobal::addWorkgroup(WorkgroupPtr workgroup)
 {
   Q_ASSERT(workgroup);
-
+  
   bool added = false;
 
-  mutex.lock();
-
-  if (!findWorkgroup(workgroup->workgroupName()))
+  if (workgroup)
   {
-    p->workgroupsList.append(workgroup);
-    added = true;
-  }
-  else
-  {
-    // Do nothing
-  }
+    mutex.lock();
 
-  mutex.unlock();
-
-  return added;
-}
-
-
-bool Smb4KGlobal::removeWorkgroup(Smb4KWorkgroup *workgroup)
-{
-  Q_ASSERT(workgroup);
-
-  bool removed = false;
-
-  mutex.lock();
-
-  int index = p->workgroupsList.indexOf(workgroup);
-
-  if (index != -1)
-  {
-    // The workgroup was found. Remove it.
-    delete p->workgroupsList.takeAt(index);
-    removed = true;
-  }
-  else
-  {
-    // Try harder to find the workgroup.
-    Smb4KWorkgroup *wg = findWorkgroup(workgroup->workgroupName());
-
-    if (wg)
+    if (!findWorkgroup(workgroup->workgroupName()))
     {
-      index = p->workgroupsList.indexOf(wg);
-
-      if (index != -1)
-      {
-        delete p->workgroupsList.takeAt(index);
-        removed = true;
-      }
-      else
-      {
-        // Do nothing
-      }
+      p->workgroupsList.append(workgroup);
+      added = true;
     }
     else
     {
       // Do nothing
     }
 
-    delete workgroup;
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
   }
 
-  mutex.unlock();
+  return added;
+}
+
+
+bool Smb4KGlobal::updateWorkgroup(WorkgroupPtr workgroup)
+{
+  Q_ASSERT(workgroup);
+  
+  bool updated = false;
+  
+  if (workgroup)
+  {
+    mutex.lock();
+    
+    WorkgroupPtr existingWorkgroup = findWorkgroup(workgroup->workgroupName());
+    
+    if (existingWorkgroup)
+    {
+      existingWorkgroup->update(workgroup.data());
+      updated = true;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  return updated;
+}
+
+
+
+bool Smb4KGlobal::removeWorkgroup(WorkgroupPtr workgroup)
+{
+  Q_ASSERT(workgroup);
+
+  bool removed = false;
+  
+  if (workgroup)
+  {
+    mutex.lock();
+
+    int index = p->workgroupsList.indexOf(workgroup);
+
+    if (index != -1)
+    {
+      // The workgroup was found. Remove it.
+      p->workgroupsList.takeAt(index).clear();
+      removed = true;
+    }
+    else
+    {
+      // Try harder to find the workgroup.
+      WorkgroupPtr wg = findWorkgroup(workgroup->workgroupName());
+
+      if (wg)
+      {
+        index = p->workgroupsList.indexOf(wg);
+
+        if (index != -1)
+        {
+          p->workgroupsList.takeAt(index).clear();
+          removed = true;
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        // Do nothing
+      }
+
+      workgroup.clear();
+    }
+
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
+  }
 
   return removed;
 }
@@ -238,26 +288,26 @@ void Smb4KGlobal::clearWorkgroupsList()
 
   while (!p->workgroupsList.isEmpty())
   {
-    delete p->workgroupsList.takeFirst();
+    p->workgroupsList.takeFirst().clear();
   }
 
   mutex.unlock();
 }
 
 
-const QList<Smb4KHost *> &Smb4KGlobal::hostsList()
+const QList<HostPtr> &Smb4KGlobal::hostsList()
 {
   return p->hostsList;
 }
 
 
-Smb4KHost *Smb4KGlobal::findHost(const QString &name, const QString &workgroup)
+HostPtr Smb4KGlobal::findHost(const QString &name, const QString &workgroup)
 {
-  Smb4KHost *host = 0;
+  HostPtr host;
 
   mutex.lock();
   
-  for (Smb4KHost *h : p->hostsList)
+  for (const HostPtr &h : p->hostsList)
   {
     if ((workgroup.isEmpty() || QString::compare(h->workgroupName(), workgroup, Qt::CaseInsensitive) == 0) &&
         QString::compare(h->hostName(), name, Qt::CaseInsensitive) == 0)
@@ -277,74 +327,122 @@ Smb4KHost *Smb4KGlobal::findHost(const QString &name, const QString &workgroup)
 }
 
 
-bool Smb4KGlobal::addHost(Smb4KHost *host)
+bool Smb4KGlobal::addHost(HostPtr host)
 {
   Q_ASSERT(host);
-
+  
   bool added = false;
 
-  mutex.lock();
-
-  if (!findHost(host->hostName(), host->workgroupName()))
+  if (host)
   {
-    p->hostsList.append(host);
-    added = true;
-  }
-  else
-  {
-    // Do nothing
-  }
+    mutex.lock();
 
-  mutex.unlock();
-
-  return added;
-}
-
-
-bool Smb4KGlobal::removeHost(Smb4KHost *host)
-{
-  Q_ASSERT(host);
-
-  bool removed = false;
-
-  mutex.lock();
-
-  int index = p->hostsList.indexOf(host);
-
-  if (index != -1)
-  {
-    // The host was found. Remove it.
-    delete p->hostsList.takeAt(index);
-    removed = true;
-  }
-  else
-  {
-    // Try harder to find the host.
-    Smb4KHost *h = findHost(host->hostName(), host->workgroupName());
-
-    if (h)
+    if (!findHost(host->hostName(), host->workgroupName()))
     {
-      index = p->hostsList.indexOf(h);
-
-      if (index != -1)
-      {
-        delete p->hostsList.takeAt(index);
-        removed = true;
-      }
-      else
-      {
-        // Do nothing
-      }
+      p->hostsList.append(host);
+      added = true;
     }
     else
     {
       // Do nothing
     }
 
-    delete host;
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
   }
 
-  mutex.unlock();
+  return added;
+}
+
+
+bool Smb4KGlobal::updateHost(HostPtr host)
+{
+  Q_ASSERT(host);
+  
+  bool updated = false;
+  
+  if (host)
+  {
+    mutex.lock();
+    
+    HostPtr existingHost = findHost(host->hostName(), host->workgroupName());
+    
+    if (existingHost)
+    {
+      existingHost->update(host.data());
+      updated = true;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  return updated;
+}
+
+
+
+bool Smb4KGlobal::removeHost(HostPtr host)
+{
+  Q_ASSERT(host);
+
+  bool removed = false;
+  
+  if (host)
+  {
+    mutex.lock();
+
+    int index = p->hostsList.indexOf(host);
+
+    if (index != -1)
+    {
+      // The host was found. Remove it.
+      p->hostsList.takeAt(index).clear();
+      removed = true;
+    }
+    else
+    {
+      // Try harder to find the host.
+      HostPtr h = findHost(host->hostName(), host->workgroupName());
+
+      if (h)
+      {
+        index = p->hostsList.indexOf(h);
+
+        if (index != -1)
+        {
+          p->hostsList.takeAt(index).clear();
+          removed = true;
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        // Do nothing
+      }
+
+      host.clear();
+    }
+
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
+  }
 
   return removed;
 }
@@ -356,20 +454,20 @@ void Smb4KGlobal::clearHostsList()
 
   while (!p->hostsList.isEmpty())
   {
-    delete p->hostsList.takeFirst();
+    p->hostsList.takeFirst().clear();
   }
 
   mutex.unlock();
 }
 
 
-QList<Smb4KHost *> Smb4KGlobal::workgroupMembers(Smb4KWorkgroup *workgroup)
+QList<HostPtr> Smb4KGlobal::workgroupMembers(WorkgroupPtr workgroup)
 {
-  QList<Smb4KHost *> hosts;
+  QList<HostPtr> hosts;
 
   mutex.lock();
   
-  for (Smb4KHost *h : p->hostsList)
+  for (const HostPtr &h : p->hostsList)
   {
     if (QString::compare(h->workgroupName(), workgroup->workgroupName(), Qt::CaseInsensitive) == 0)
     {
@@ -387,19 +485,19 @@ QList<Smb4KHost *> Smb4KGlobal::workgroupMembers(Smb4KWorkgroup *workgroup)
 }
 
 
-const QList<Smb4KShare *> &Smb4KGlobal::sharesList()
+const QList<SharePtr> &Smb4KGlobal::sharesList()
 {
   return p->sharesList;
 }
 
 
-Smb4KShare *Smb4KGlobal::findShare(const QString& unc, const QString& workgroup)
+SharePtr Smb4KGlobal::findShare(const QString& unc, const QString& workgroup)
 {
-  Smb4KShare *share = 0;
+  SharePtr share;
   
   mutex.lock();
 
-  for (Smb4KShare *s : p->sharesList)
+  for (const SharePtr &s : p->sharesList)
   {
     if (QString::compare(s->unc(), unc, Qt::CaseInsensitive) == 0 &&
         (workgroup.isEmpty() || QString::compare(s->workgroupName(), workgroup, Qt::CaseInsensitive) == 0))
@@ -420,7 +518,7 @@ Smb4KShare *Smb4KGlobal::findShare(const QString& unc, const QString& workgroup)
 
 
 
-bool Smb4KGlobal::addShare(Smb4KShare *share)
+bool Smb4KGlobal::addShare(SharePtr share)
 {
   Q_ASSERT(share);
 
@@ -429,9 +527,41 @@ bool Smb4KGlobal::addShare(Smb4KShare *share)
   if (share)
   {
     mutex.lock();
-
+    
+    //
+    // Add the share
+    //
     if (!findShare(share->unc(), share->workgroupName()))
     {
+      // 
+      // Set the share mounted
+      // Only honor shares that are owned by the user
+      // 
+      QList<SharePtr> mountedShares = findShareByUNC(share->unc());
+      
+      if (!mountedShares.isEmpty())
+      {
+        for (const SharePtr &s : mountedShares)
+        {
+          if (!s->isForeign())
+          {
+            share->setMountData(s.data());
+            break;
+          }
+          else
+          {
+            continue;
+          }
+        }
+      }
+      else
+      {
+        // Do nothing
+      }  
+      
+      // 
+      // Add it
+      // 
       p->sharesList.append(share);
       added = true;
     }
@@ -451,7 +581,73 @@ bool Smb4KGlobal::addShare(Smb4KShare *share)
 }
 
 
-bool Smb4KGlobal::removeShare(Smb4KShare *share)
+bool Smb4KGlobal::updateShare(SharePtr share)
+{
+  Q_ASSERT(share);
+  
+  bool updated = false;
+  
+  if (share)
+  {
+    mutex.lock();
+    
+    //
+    // Updated the share
+    //
+    SharePtr existingShare = findShare(share->unc(), share->workgroupName());
+    
+    if (existingShare)
+    {
+      // 
+      // Set the share mounted
+      // Only honor shares that are owned by the user
+      // 
+      QList<SharePtr> mountedShares = findShareByUNC(share->unc());
+      
+      if (!mountedShares.isEmpty())
+      {
+        for (const SharePtr &s : mountedShares)
+        {
+          if (!s->isForeign())
+          {
+            share->setMountData(s.data());
+            break;
+          }
+          else
+          {
+            continue;
+          }
+        }
+      }
+      else
+      {
+        // Do nothing
+      }
+      
+      // 
+      // Update it
+      // 
+      existingShare->update(share.data());
+      updated = true;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    mutex.unlock();
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  return updated;
+}
+
+
+
+bool Smb4KGlobal::removeShare(SharePtr share)
 {
   Q_ASSERT(share);
 
@@ -466,13 +662,13 @@ bool Smb4KGlobal::removeShare(Smb4KShare *share)
     if (index != -1)
     {
       // The share was found. Remove it.
-      delete p->sharesList.takeAt(index);
+      p->sharesList.takeAt(index).clear();
       removed = true;
     }
     else
     {
       // Try harder to find the share.
-      Smb4KShare *s = findShare(share->unc(), share->workgroupName());
+      SharePtr s = findShare(share->unc(), share->workgroupName());
 
       if (s)
       {
@@ -480,7 +676,7 @@ bool Smb4KGlobal::removeShare(Smb4KShare *share)
 
         if (index != -1)
         {
-          delete p->sharesList.takeAt(index);
+          p->sharesList.takeAt(index).clear();
           removed = true;
         }
         else
@@ -493,7 +689,7 @@ bool Smb4KGlobal::removeShare(Smb4KShare *share)
         // Do nothing
       }
 
-      delete share;
+      share.clear();
     }
 
     mutex.unlock();
@@ -513,20 +709,20 @@ void Smb4KGlobal::clearSharesList()
 
   while (!p->sharesList.isEmpty())
   {
-    delete p->sharesList.takeFirst();
+    p->sharesList.takeFirst().clear();
   }
 
   mutex.unlock();
 }
 
 
-QList<Smb4KShare *> Smb4KGlobal::sharedResources(Smb4KHost *host)
+QList<SharePtr> Smb4KGlobal::sharedResources(HostPtr host)
 {
-  QList<Smb4KShare *> shares;
+  QList<SharePtr> shares;
 
   mutex.lock();
   
-  for (Smb4KShare *s : p->sharesList)
+  for (const SharePtr &s : p->sharesList)
   {
     if (QString::compare(s->hostName(), host->hostName(), Qt::CaseInsensitive) == 0 &&
         QString::compare(s->workgroupName(), host->workgroupName(), Qt::CaseInsensitive) == 0)
@@ -545,21 +741,21 @@ QList<Smb4KShare *> Smb4KGlobal::sharedResources(Smb4KHost *host)
 }
 
 
-const QList<Smb4KShare *> &Smb4KGlobal::mountedSharesList()
+const QList<SharePtr> &Smb4KGlobal::mountedSharesList()
 {
   return p->mountedSharesList;
 }
 
 
-Smb4KShare* Smb4KGlobal::findShareByPath(const QString &path)
+SharePtr Smb4KGlobal::findShareByPath(const QString &path)
 {
-  Smb4KShare *share = 0;
+  SharePtr share;
 
   mutex.lock();
 
   if (!path.isEmpty() && !p->mountedSharesList.isEmpty())
   {
-    for (Smb4KShare *s : p->mountedSharesList)
+    for (const SharePtr &s : p->mountedSharesList)
     {
       if (QString::compare(s->path(), path, Qt::CaseInsensitive) == 0 ||
           QString::compare(s->canonicalPath(), path, Qt::CaseInsensitive) == 0)
@@ -584,15 +780,15 @@ Smb4KShare* Smb4KGlobal::findShareByPath(const QString &path)
 }
 
 
-QList<Smb4KShare *> Smb4KGlobal::findShareByUNC(const QString &unc)
+QList<SharePtr> Smb4KGlobal::findShareByUNC(const QString &unc)
 {
-  QList<Smb4KShare *> shares;
+  QList<SharePtr> shares;
 
   mutex.lock();
 
   if (!unc.isEmpty() && !p->mountedSharesList.isEmpty())
   {
-    for (Smb4KShare *s : p->mountedSharesList)
+    for (const SharePtr &s : p->mountedSharesList)
     {
       if (QString::compare(s->unc(), unc, Qt::CaseInsensitive) == 0)
       {
@@ -615,13 +811,13 @@ QList<Smb4KShare *> Smb4KGlobal::findShareByUNC(const QString &unc)
 }
 
 
-QList<Smb4KShare*> Smb4KGlobal::findInaccessibleShares()
+QList<SharePtr> Smb4KGlobal::findInaccessibleShares()
 {
-  QList<Smb4KShare *> inaccessibleShares;
+  QList<SharePtr> inaccessibleShares;
 
   mutex.lock();
   
-  for (Smb4KShare *s : p->mountedSharesList)
+  for (const SharePtr &s : p->mountedSharesList)
   {
     if (s->isInaccessible())
     {
@@ -639,7 +835,7 @@ QList<Smb4KShare*> Smb4KGlobal::findInaccessibleShares()
 }
 
 
-bool Smb4KGlobal::addMountedShare(Smb4KShare *share)
+bool Smb4KGlobal::addMountedShare(SharePtr share)
 {
   Q_ASSERT(share);
 
@@ -649,14 +845,68 @@ bool Smb4KGlobal::addMountedShare(Smb4KShare *share)
   {
     mutex.lock();
 
+    //
+    // Copy the mount data to the network share (needed for unmounting from the network browser)
+    // Only honor shares that were mounted by the user.
+    //
+    if (!share->isForeign())
+    {
+      SharePtr networkShare = findShare(share->unc(), share->workgroupName());
+      
+      if (networkShare)
+      {
+        networkShare->setMountData(share.data());
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    else
+    {
+      // Do nothing
+    }
+
     if (!findShareByPath(share->path()))
     {
+      //
+      // Check if we have to add a workgroup name and/or IP address
+      //
+      HostPtr networkHost = findHost(share->hostName(), share->workgroupName());
+      
+      if (networkHost)
+      {
+        // Set the IP address
+        if (!share->hasHostIP() || networkHost->ip() != share->hostIP())
+        {
+          share->setHostIP(networkHost->ip());
+        }
+        else
+        {
+          // Do nothing
+        }
+        
+        // Set the workgroup name
+        if (share->workgroupName().isEmpty())
+        {
+          share->setWorkgroupName(networkHost->workgroupName());
+        }
+        else
+        {
+          // Do nothing
+        }          
+      }
+      else
+      {
+        // Do nothing
+      }
+      
       p->mountedSharesList.append(share);
       added = true;
 
       p->onlyForeignShares = true;
     
-      for (Smb4KShare *s : p->mountedSharesList)
+      for (const SharePtr &s : p->mountedSharesList)
       {
         if (!s->isForeign())
         {
@@ -685,7 +935,7 @@ bool Smb4KGlobal::addMountedShare(Smb4KShare *share)
 }
 
 
-bool Smb4KGlobal::updateMountedShare(Smb4KShare* share)
+bool Smb4KGlobal::updateMountedShare(SharePtr share)
 {
   Q_ASSERT(share);
   
@@ -693,18 +943,79 @@ bool Smb4KGlobal::updateMountedShare(Smb4KShare* share)
   
   if (share)
   {
-    Smb4KShare *mountedShare = findShareByPath(share->path());
+    mutex.lock();
+    
+    //
+    // Copy the mount data to the network share (needed for unmounting from the network browser)
+    // Only honor shares that were mounted by the user.
+    //
+    if (!share->isForeign())
+    {
+      SharePtr networkShare = findShare(share->unc(), share->workgroupName());
+      
+      if (networkShare)
+      {
+        networkShare->setMountData(share.data());
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    SharePtr mountedShare = findShareByPath(share->path());
     
     if (mountedShare)
     {
+      //
+      // Check if we have to add a workgroup name and/or IP address
+      //
+      HostPtr networkHost = findHost(share->hostName(), share->workgroupName());
+      
+      if (networkHost)
+      {
+        // Set the IP address
+        if (!share->hasHostIP() || networkHost->ip() != share->hostIP())
+        {
+          share->setHostIP(networkHost->ip());
+        }
+        else
+        {
+          // Do nothing
+        }
+        
+        // Set the workgroup name
+        if (share->workgroupName().isEmpty())
+        {
+          share->setWorkgroupName(networkHost->workgroupName());
+        }
+        else
+        {
+          // Do nothing
+        }          
+      }
+      else
+      {
+        // Do nothing
+      }
+      
+      
+      //
       // Update share
-      mountedShare->setMountData(share);
+      //
+      mountedShare->setMountData(share.data());
       updated = true;
     }
     else
     {
       // Do nothing
     }
+    
+    mutex.unlock();
   }
   else
   {
@@ -715,7 +1026,7 @@ bool Smb4KGlobal::updateMountedShare(Smb4KShare* share)
 }
 
 
-bool Smb4KGlobal::removeMountedShare(Smb4KShare *share)
+bool Smb4KGlobal::removeMountedShare(SharePtr share)
 {
   Q_ASSERT(share);
 
@@ -724,28 +1035,54 @@ bool Smb4KGlobal::removeMountedShare(Smb4KShare *share)
   if (share)
   {
     mutex.lock();
-  
-    // Find the share by its path and remove it.
-    QMutableListIterator<Smb4KShare *> it(p->mountedSharesList);
     
-    while (it.hasNext())
+    SharePtr networkShare = findShare(share->unc(), share->workgroupName());
+    
+    if (networkShare)
     {
-      Smb4KShare *s = it.next();
-      
-      if (QString::compare(s->path(), share->path(), Qt::CaseInsensitive) == 0 ||
-          QString::compare(s->canonicalPath(), share->canonicalPath(), Qt::CaseInsensitive) == 0)
+      networkShare->resetMountData();
+    }
+    else
+    {
+      // Do nothing
+    }
+
+    int index = p->mountedSharesList.indexOf(share);
+
+    if (index != -1)
+    {
+      // The share was found. Remove it.
+      p->mountedSharesList.takeAt(index).clear();
+      removed = true;
+    }
+    else
+    {
+      // Try harder to find the share.
+      SharePtr s = findShareByPath(share->isInaccessible() ? share->path() : share->canonicalPath());
+
+      if (s)
       {
-        it.remove();
-        removed = true;
-        break;
+        index = p->mountedSharesList.indexOf(s);
+
+        if (index != -1)
+        {
+          p->mountedSharesList.takeAt(index).clear();
+          removed = true;
+        }
+        else
+        {
+          // Do nothing
+        }
       }
       else
       {
         // Do nothing
       }
+
+      share.clear();
     }
     
-    for (Smb4KShare *s : p->mountedSharesList)
+    for (const SharePtr &s : p->mountedSharesList)
     {
       if (!s->isForeign())
       {
@@ -758,8 +1095,6 @@ bool Smb4KGlobal::removeMountedShare(Smb4KShare *share)
       }
     }
     
-    delete share;
-    
     mutex.unlock();
   }
   else
@@ -777,7 +1112,7 @@ bool Smb4KGlobal::onlyForeignMountedShares()
 }
 
 
-void Smb4KGlobal::openShare(Smb4KShare *share, OpenWith openWith)
+void Smb4KGlobal::openShare(SharePtr share, OpenWith openWith)
 {
   if (!share || share->isInaccessible())
   {
@@ -896,3 +1231,10 @@ const QString Smb4KGlobal::findUmountExecutable()
 
   return QStandardPaths::findExecutable("umount", paths);
 }
+
+
+const QString Smb4KGlobal::dataLocation()
+{
+  return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation)+QDir::separator()+"smb4k";
+}
+
