@@ -31,7 +31,6 @@
 #include "smb4kmounter.h"
 #include "smb4kmounter_p.h"
 #include "smb4kauthinfo.h"
-#include "smb4kglobal.h"
 #include "smb4kshare.h"
 #include "smb4ksettings.h"
 #include "smb4khomesshareshandler.h"
@@ -101,18 +100,17 @@ Smb4KMounter::Smb4KMounter(QObject *parent)
   d->remountAttempts = 0;
   d->checkTimeout = 0;
   d->newlyMounted = 0;
+  d->newlyUnmounted = 0;
   d->dialog = 0;
   d->firstImportDone = false;
-  d->aboutToQuit = false;
+  d->mountShares = false;
+  d->unmountShares = false;
   d->activeProfile = Smb4KProfileManager::self()->activeProfile();
   d->detectAllShares = Smb4KSettings::detectAllShares();
 
   connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
           this, SLOT(slotAboutToQuit()));
   
-  connect(Smb4KHardwareInterface::self(), SIGNAL(networkConfigUpdated()), 
-          this, SLOT(slotStartJobs()));
-  
   connect(Smb4KHardwareInterface::self(), SIGNAL(onlineStateChanged(bool)),
           this, SLOT(slotOnlineStateChanged(bool)));
   
@@ -125,6 +123,9 @@ Smb4KMounter::Smb4KMounter(QObject *parent)
   connect(Smb4KProfileManager::self(), SIGNAL(migratedProfile(QString,QString)),
           this, SLOT(slotProfileMigrated(QString,QString)));
   
+  connect(Smb4KProfileManager::self(), SIGNAL(aboutToChangeProfile()),
+          this, SLOT(slotAboutToChangeProfile()));
+  
   connect(Smb4KProfileManager::self(), SIGNAL(activeProfileChanged(QString)),
           this, SLOT(slotActiveProfileChanged(QString)));
   
@@ -137,12 +138,12 @@ Smb4KMounter::~Smb4KMounter()
 {
   while (!d->importedShares.isEmpty())
   {
-    delete d->importedShares.takeFirst();
+    d->importedShares.takeFirst().clear();
   }
 
   while (!d->retries.isEmpty())
   {
-    delete d->retries.takeFirst();
+    d->retries.takeFirst().clear();
   }
 }
 
@@ -184,23 +185,27 @@ void Smb4KMounter::triggerRemounts(bool fill_list)
   {
     if (fill_list)
     {
+      //
       // Get the shares that are to be remounted
-      QList<Smb4KCustomOptions *> list = Smb4KCustomOptionsManager::self()->sharesToRemount();
+      //
+      QList<OptionsPtr> list = Smb4KCustomOptionsManager::self()->sharesToRemount();
 
       if (!list.isEmpty())
       {
-        // Check which ones actually need to be remounted.
-        for (int i = 0; i < list.size(); ++i)
+        //
+        // Check which shares actually need to be remounted
+        //
+        for (const OptionsPtr &opt : list)
         {
-          QList<Smb4KShare *> mounted_shares = findShareByUNC(list.at(i)->unc());
-
-          if (!mounted_shares.isEmpty())
+          QList<SharePtr> mountedShares = findShareByUNC(opt->unc());
+          
+          if (!mountedShares.isEmpty())
           {
             bool mount = true;
-
-            for (int j = 0; j < mounted_shares.size(); ++j)
+            
+            for (const SharePtr &s : mountedShares)
             {
-              if (!mounted_shares.at(j)->isForeign())
+              if (!s->isForeign())
               {
                 mount = false;
                 break;
@@ -210,15 +215,15 @@ void Smb4KMounter::triggerRemounts(bool fill_list)
                 continue;
               }
             }
-
+            
             if (mount)
             {
-              Smb4KShare *share = new Smb4KShare();
-              share->setURL(list.at(i)->url());
-              share->setWorkgroupName(list.at(i)->workgroupName());
-              share->setHostIP(list.at(i)->ip());
-
-              if (!share->url().isEmpty())
+              SharePtr share = SharePtr(new Smb4KShare());
+              share->setURL(opt->url());
+              share->setWorkgroupName(opt->workgroupName());
+              share->setHostIP(opt->ip());
+              
+              if (share->url().isValid() && !share->url().isEmpty())
               {
                 d->remounts << share;
               }
@@ -234,12 +239,12 @@ void Smb4KMounter::triggerRemounts(bool fill_list)
           }
           else
           {
-            Smb4KShare *share = new Smb4KShare();
-            share->setURL(list.at(i)->url());
-            share->setWorkgroupName(list.at(i)->workgroupName());
-            share->setHostIP(list.at(i)->ip());
-
-            if (!share->url().isEmpty())
+            SharePtr share = SharePtr(new Smb4KShare());
+            share->setURL(opt->url());
+            share->setWorkgroupName(opt->workgroupName());
+            share->setHostIP(opt->ip());
+              
+            if (share->url().isValid() && !share->url().isEmpty())
             {
               d->remounts << share;
             }
@@ -304,7 +309,7 @@ void Smb4KMounter::import(bool checkInaccessible)
     if (QString::compare(mountPoint->mountType(), "cifs") == 0 || QString::compare(mountPoint->mountType(), "smbfs") == 0)
     {
       // Create new share and set the mountpoint and the filesystem
-      Smb4KShare *share = new Smb4KShare(mountPoint->mountedFrom());
+      SharePtr share = SharePtr(new Smb4KShare(mountPoint->mountedFrom()));
       share->setPath(mountPoint->mountPoint());
       share->setMounted(true);
       
@@ -354,12 +359,12 @@ void Smb4KMounter::import(bool checkInaccessible)
   // NOTE: The unmount() signal is emitted *BEFORE* the share is removed
   // from the global list! You need to account for that in your application.
   //
-  QList<Smb4KShare *> unmountedShares;
+  QList<SharePtr> unmountedShares;
   bool found = false;
   
-  for (Smb4KShare *mountedShare : mountedSharesList())
+  for (const SharePtr &mountedShare : mountedSharesList())
   {
-    for (Smb4KShare *importedShare : d->importedShares)
+    for (const SharePtr &importedShare : d->importedShares)
     {
       // Check the mountpoint, since that one is unique. We will only use
       // Smb4KShare::path(), so that we do not run into trouble if a share 
@@ -411,30 +416,59 @@ void Smb4KMounter::import(bool checkInaccessible)
   
   if (!unmountedShares.isEmpty())
   {
-    if (unmountedShares.size() > 1)
+    d->newlyUnmounted += unmountedShares.size();
+    
+    if (!d->mountShares && !d->unmountShares)
     {
-      int size = unmountedShares.size();
-      
-      for (Smb4KShare *share : unmountedShares)
+      if (d->newlyUnmounted == 1)
       {
-        emit unmounted(share);
-        removeMountedShare(share);
+        // Copy the share
+        SharePtr unmountedShare = unmountedShares.first();
+        
+        // Remove the share from the global list and notify the program and user
+        removeMountedShare(unmountedShares.first());
+        emit unmounted(unmountedShare);
+        Smb4KNotification::shareUnmounted(unmountedShare);
+        unmountedShare.clear();
+      }
+      else
+      {
+        for (const SharePtr &share : unmountedShares)
+        {
+          // Copy the share
+          SharePtr unmountedShare = share;
+          
+          // Remove the share from the global list and notify the program
+          removeMountedShare(share);
+          emit unmounted(unmountedShare);
+          unmountedShare.clear();
+        }
+        
+        // Notify the user
+        Smb4KNotification::sharesUnmounted(d->newlyUnmounted);
       }
       
-      Smb4KNotification::sharesUnmounted(size);
+      d->newlyUnmounted = 0;
     }
     else
     {
-      emit unmounted(unmountedShares.first());
-      Smb4KNotification::shareUnmounted(unmountedShares.first());
-      removeMountedShare(unmountedShares.first());
+      for (const SharePtr &share : unmountedShares)
+      {
+        // Copy the share
+        SharePtr unmountedShare = share;
+        
+        // Remove the share from the global list and notify the program
+        removeMountedShare(share);
+        emit unmounted(unmountedShare);
+        unmountedShare.clear();
+      }
     }
     
     emit mountedSharesListChanged();
   }
   else
   {
-    // Do nothing
+    d->newlyUnmounted = 0;
   }
   
   //
@@ -444,14 +478,18 @@ void Smb4KMounter::import(bool checkInaccessible)
   //
   if (Smb4KHardwareInterface::self()->isOnline())
   {
-    for (Smb4KShare *share : d->importedShares)
+    QMutableListIterator<SharePtr> it(d->importedShares);
+    
+    while (it.hasNext())
     {
-      Smb4KShare *mountedShare = findShareByPath(share->path());
+      SharePtr share = it.next();
+      SharePtr mountedShare = findShareByPath(share->path());
       
       if (mountedShare)
       {
         if (mountedShare->isInaccessible() && !checkInaccessible)
         {
+          it.remove();
           continue;
         }
         else
@@ -468,8 +506,9 @@ void Smb4KMounter::import(bool checkInaccessible)
       KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
       job->setDetails(0);
       connect(job, SIGNAL(result(KJob*)), this, SLOT(slotStatResult(KJob*)));
-      
+       
       // Do not use addSubJob(), because that would confuse isRunning(), etc.
+      
       job->start();
     }
     
@@ -489,31 +528,14 @@ void Smb4KMounter::import(bool checkInaccessible)
 }
 
 
-void Smb4KMounter::mountShare(Smb4KShare *share, QWidget *parent)
+void Smb4KMounter::mountShare(const SharePtr &share, QWidget *parent)
 {
   Q_ASSERT(share);
   
   if (share)
-  {
-    QList<Smb4KShare *> shares;
-    shares << share;
-    mountShares(shares, parent);
-  }
-  else
-  {
-    // Do nothing
-  }  
-}
-
-
-void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *parent)
-{
-  QVariantMap map;
-  
-  for (Smb4KShare *share : shares)
   {
     //
-    // Check that the URL is valid.
+    // Check that the URL is valid
     //
     if (!share->url().isValid())
     {
@@ -527,7 +549,7 @@ void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *paren
     
     //
     // Check if the share has already been mounted. If it is already present,
-    // do not process this share and move on to the next.
+    // do not process it and return.
     //
     QString unc;
     
@@ -535,7 +557,7 @@ void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *paren
     {
       if (!Smb4KHomesSharesHandler::self()->specifyUser(share, true, parent))
       {
-        continue;
+        return;
       }
       else
       {
@@ -549,14 +571,14 @@ void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *paren
       unc = share->unc();
     }
     
-    QList<Smb4KShare *> mountedShares = findShareByUNC(unc);
-    bool mounted = false;
-    
-    for (Smb4KShare *s : mountedShares)
+    QList<SharePtr> mountedShares = findShareByUNC(unc);
+    bool isMounted = false;
+      
+    for (const SharePtr &s : mountedShares)
     {
       if (!s->isForeign())
       {
-        mounted = true;
+        isMounted = true;
       }
       else
       {
@@ -564,21 +586,21 @@ void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *paren
       }
     }
     
-    if (mounted)
+    if (isMounted)
     {
-      continue;
+      return;
     }
     else
     {
       // Do nothing
     }
-    
+
     //
     // Wake-On-LAN: Wake up the host before mounting 
     //
     if (Smb4KSettings::enableWakeOnLAN())
     {
-      Smb4KCustomOptions *options = Smb4KCustomOptionsManager::self()->findOptions(KIO::upUrl(share->url()));
+      OptionsPtr options = Smb4KCustomOptionsManager::self()->findOptions(KIO::upUrl(share->url()));
 
       if (options && options->wolSendBeforeMount())
       {
@@ -705,7 +727,7 @@ void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *paren
     // Get the authentication information
     //
     Smb4KWalletManager::self()->readAuthInfo(share);
- 
+
     //
     // Mount arguments
     //
@@ -713,79 +735,164 @@ void Smb4KMounter::mountShares(const QList<Smb4KShare *> &shares, QWidget *paren
     
     if (!fillMountActionArgs(share, args))
     {
-      continue;
+      return;
     }
     else
     {
       // Do nothing
     }
     
-    map.insert(QString("%1").arg(map.size()), args);
-  }
-  
-  //
-  // Emit the aboutToStart() signal
-  //
-  emit aboutToStart(MountShare);
+    //
+    // Emit the aboutToStart() signal
+    //
+    emit aboutToStart(MountShare);
 
-  //
-  // Create the mount action
-  //
-  KAuth::Action mountAction("org.kde.smb4k.mounthelper.mount");
-  mountAction.setHelperId("org.kde.smb4k.mounthelper");
-  mountAction.setArguments(map);
-  
-  KAuth::ExecuteJob *job = mountAction.execute();
-  job->setObjectName("MountJob");
+    //
+    // Create the mount action
+    //
+    KAuth::Action mountAction("org.kde.smb4k.mounthelper.mount");
+    mountAction.setHelperId("org.kde.smb4k.mounthelper");
+    mountAction.setArguments(args);
     
-  connect(job, SIGNAL(finished(KJob*)), this, SLOT(slotJobFinished(KJob*)));
+    KAuth::ExecuteJob *job = mountAction.execute();
+      
+    //
+    // Modify the cursor, if necessary.
+    //
+    if (!hasSubjobs() && modifyCursor())
+    {
+      QApplication::setOverrideCursor(Qt::BusyCursor);
+    }
+    else
+    {
+      // Do nothing
+    }
+      
+    //
+    // Add the job
+    //
+    addSubjob(job);
+
+    //
+    // Start the job and process the returned result.
+    //
+    bool success = job->exec();
     
-  //
-  // Modify the cursor, if necessary.
-  //
-  if (!hasSubjobs() && modifyCursor())
-  {
-    QApplication::setOverrideCursor(Qt::BusyCursor);
+    if (success)
+    {
+      int errorCode = job->error();
+      
+      if (errorCode == 0)
+      {
+        // Get the error message
+        QString errorMsg = job->data()["mh_error_message"].toString();
+        
+        if (!errorMsg.isEmpty())
+        {
+#if defined(Q_OS_LINUX)
+          if (errorMsg.contains("mount error 13") || errorMsg.contains("mount error(13)") /* authentication error */)
+          {
+            if (Smb4KWalletManager::self()->showPasswordDialog(share, 0))
+            {
+              d->retries << share;
+            }
+            else
+            {
+              // Do nothing
+            }
+          }
+          else if (errorMsg.contains("Unable to find suitable address."))
+          {
+            // Swallow this
+          }
+          else
+          {
+            Smb4KNotification::mountingFailed(share, errorMsg);
+          }
+#elif defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
+          if (errorMsg.contains("Authentication error") || errorMsg.contains("Permission denied"))
+          {
+            if (Smb4KWalletManager::self()->showPasswordDialog(share, 0))
+            {
+              d->retries << share;
+            }
+            else
+            {
+              // Do nothing
+            }
+          }
+          else
+          {
+            Smb4KNotification::mountingFailed(share, errorMsg);
+          }
+#else
+          qWarning() << "Smb4KMounter::slotMountJobFinished(): Error handling not implemented!";
+          Smb4KNotification::mountingFailed(share.data(), errorMsg);
+#endif
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        Smb4KNotification::actionFailed(errorCode);
+      }
+    }
+    else
+    {
+      // FIXME: Report that the action could not be started
+      // Do nothing
+    }
+    
+    //
+    // Remove the job from the job list
+    //
+    removeSubjob(job);
+      
+    //
+    // Reset the busy cursor
+    //
+    if (!hasSubjobs() && modifyCursor())
+    {
+      QApplication::restoreOverrideCursor();
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    //
+    // Emit the finished() signal
+    //
+    emit finished(MountShare);
   }
   else
   {
     // Do nothing
   }
-    
-  //
-  // Add the job
-  //
-  addSubjob(job);
-
-  //
-  // Start the job
-  //
-  job->start(); 
 }
 
 
-void Smb4KMounter::unmountShare(Smb4KShare *share, bool silent, QWidget *parent)
+void Smb4KMounter::mountShares(const QList<SharePtr> &shares, QWidget *parent)
+{
+  d->mountShares = true;
+  
+  for (const SharePtr &share : shares)
+  {
+    mountShare(share, parent);
+  }
+  
+  d->mountShares = false;
+}
+
+
+void Smb4KMounter::unmountShare(const SharePtr &share, bool silent, QWidget *parent)
 {
   Q_ASSERT(share);
   
   if (share)
-  {
-    QList<Smb4KShare *> shares;
-    shares << share;
-    unmountShares(shares, silent, parent);
-  }
-  else
-  {
-    // Do nothing
-  }
-}
-
-
-void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent, QWidget *parent)
-{
-  QVariantMap map;
-  
-  for (Smb4KShare *share : shares)
   {
     //
     // Check that the URL is valid.
@@ -815,7 +922,7 @@ void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent,
         {
           // Do nothing
         }
-        continue;
+        return;
       }
       else
       {
@@ -827,7 +934,7 @@ void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent,
                   share->unc(), share->path(), share->user().loginName()),
               i18n("Foreign Share")) == KMessageBox::No)
           {
-            continue;
+            return;
           }
           else
           {
@@ -838,7 +945,7 @@ void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent,
         {
           // Without the confirmation of the user, we are not
           // unmounting a foreign share!
-          continue;
+          return;
         }
       }
     }
@@ -847,9 +954,8 @@ void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent,
       // Do nothing
     }
     
-    
     //
-    // Force the unmounting of the shares either if the system went offline
+    // Force the unmounting of the share either if the system went offline
     // or if the user chose to forcibly unmount inaccessible shares (Linux only).
     //
     bool force = false;
@@ -871,7 +977,7 @@ void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent,
     {
       force = true;
     }
-
+    
     //
     // Unmount arguments
     //
@@ -879,58 +985,143 @@ void Smb4KMounter::unmountShares(const QList<Smb4KShare *> &shares, bool silent,
 
     if (!fillUnmountActionArgs(share, force, silent, args))
     {
-      continue;
+      return;
     }
     else
     {
       // Do nothing
     }
-
-    map.insert(QString("%1").arg(map.size()), args);
-  }
-  
-  // Should the unmount helper function unmount all shares at once or 
-  // loop through the whole list.
-  map.insert("mh_unmount_at_once", !Smb4KHardwareInterface::self()->isOnline() || d->aboutToQuit);
-  
-  //
-  // Emit the aboutToStart() signal
-  //
-  emit aboutToStart(UnmountShare);
-
-  //
-  // Create the unmount action
-  //
-  KAuth::Action unmountAction("org.kde.smb4k.mounthelper.unmount");
-  unmountAction.setHelperId("org.kde.smb4k.mounthelper");
-  unmountAction.setArguments(map);
     
-  KAuth::ExecuteJob *job = unmountAction.execute();
-  job->setObjectName("UnmountJob");
+    //
+    // Emit the aboutToStart() signal
+    //
+    emit aboutToStart(UnmountShare);
+
+    //
+    // Create the unmount action
+    //
+    KAuth::Action unmountAction("org.kde.smb4k.mounthelper.unmount");
+    unmountAction.setHelperId("org.kde.smb4k.mounthelper");
+    unmountAction.setArguments(args);
+      
+    KAuth::ExecuteJob *job = unmountAction.execute();
     
-  connect(job, SIGNAL(finished(KJob*)), this, SLOT(slotJobFinished(KJob*)));
-  
-  //
-  // Modify the cursor, if necessary.
-  //
-  if (!hasSubjobs() && modifyCursor())
-  {
-    QApplication::setOverrideCursor(Qt::BusyCursor);
+    //
+    // Modify the cursor, if necessary.
+    //
+    if (!hasSubjobs() && modifyCursor())
+    {
+      QApplication::setOverrideCursor(Qt::BusyCursor);
+    }
+    else
+    {
+      // Do nothing
+    }
+      
+    //
+    // Add the job
+    //
+    addSubjob(job);
+
+    //
+    // Start the job and process the returned result.
+    //
+    bool success = job->exec();
+    
+    if (success)
+    {
+      int errorCode = job->error();
+      
+      if (errorCode == 0)
+      {
+        // Get the error message
+        QString errorMsg = job->data()["mh_error_message"].toString();
+        
+        if (!errorMsg.isEmpty())
+        {
+          // No error handling needed, just report the error message.
+          Smb4KNotification::unmountingFailed(share, errorMsg);
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        Smb4KNotification::actionFailed(errorCode);
+      }
+
+    }
+    else
+    {
+      // FIXME: Report that the action could not be started
+      // Do nothing
+    }
+    
+    //
+    // Remove the job from the job list
+    //
+    removeSubjob(job);
+      
+    //
+    // Reset the busy cursor
+    //
+    if (!hasSubjobs() && modifyCursor())
+    {
+      QApplication::restoreOverrideCursor();
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    //
+    // Emit the finished() signal
+    //
+    emit finished(UnmountShare);
   }
   else
   {
     // Do nothing
   }
-    
-  //
-  // Add the job
-  //
-  addSubjob(job);
+}
 
+
+void Smb4KMounter::unmountShares(const QList<SharePtr> &shares, bool silent, QWidget *parent)
+{
+#if defined(Q_OS_LINUX)
   //
-  // Start the job
+  // Under Linux, we have to take an approach that is a bit awkward in this function. 
+  // Since the import function is invoked via Smb4KHardwareInterface::networkShareRemoved()
+  // before mountShare() returns, no unmount of multiple shares will ever be reported
+  // when d->unmountShares is set to FALSE *after* the loop ended. To make the reporting
+  // work correctly, we need to set d->unmountShares to FALSE before the last unmount
+  // is started.
   //
-  job->start();  
+  d->unmountShares = true;
+  int number = shares.size();
+  
+  for (const SharePtr &share : shares)
+  {
+    number--;
+    d->unmountShares = (number != 0);
+    unmountShare(share, silent, parent);
+  }
+#elif defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
+  //
+  // Since under FreeBSD the emission of Smb4KHardwareInterface::networkShareRemoved() is 
+  // triggered by a timer, we can use a nice approach here.
+  //
+  d->unmountShares = true;
+  
+  for (const SharePtr &share : shares)
+  {
+    unmountShare(share, silent, parent);
+  }
+  
+  d->unmountShares = false;
+#endif
 }
 
 
@@ -944,7 +1135,7 @@ void Smb4KMounter::openMountDialog(QWidget *parent)
 {
   if (!d->dialog)
   {
-    Smb4KShare *share = new Smb4KShare();
+    SharePtr share = SharePtr(new Smb4KShare());
     
     d->dialog = new Smb4KMountDialog(share, parent);
 
@@ -971,7 +1162,7 @@ void Smb4KMounter::openMountDialog(QWidget *parent)
     delete d->dialog;
     d->dialog = 0;
 
-    delete share;
+    share.clear();
   }
   else
   {
@@ -986,6 +1177,11 @@ void Smb4KMounter::start()
   // Check the network configurations
   //
   Smb4KHardwareInterface::self()->updateNetworkConfig();
+  
+  //
+  // Connect to Smb4KHardwareInterface to be able to get the response
+  // 
+  connect(Smb4KHardwareInterface::self(), SIGNAL(networkConfigUpdated()), this, SLOT(slotStartJobs()));
 }
 
 
@@ -994,7 +1190,7 @@ void Smb4KMounter::saveSharesForRemount()
   //
   // Save the shares for remount
   //
-  for (Smb4KShare *share : mountedSharesList())
+  for (const SharePtr &share : mountedSharesList())
   {
     if (!share->isForeign())
     {
@@ -1011,9 +1207,9 @@ void Smb4KMounter::saveSharesForRemount()
   //
   while (!d->remounts.isEmpty())
   {
-    Smb4KShare *share = d->remounts.takeFirst();
+    SharePtr share = d->remounts.takeFirst();
     Smb4KCustomOptionsManager::self()->addRemount(share, false);
-    delete share;
+    share.clear();
   }  
 }
 
@@ -1027,7 +1223,7 @@ void Smb4KMounter::timerEvent(QTimerEvent *)
   if ((Smb4KSettings::remountShares() || !Smb4KCustomOptionsManager::self()->sharesToRemount().isEmpty()) && 
        Smb4KSettings::remountAttempts() > d->remountAttempts)
   {
-    if (d->firstImportDone && !hasSubjobs())
+    if (d->firstImportDone && !isRunning())
     {
       if (d->remountAttempts == 0)
       {
@@ -1063,7 +1259,7 @@ void Smb4KMounter::timerEvent(QTimerEvent *)
       
     while (!d->retries.isEmpty())
     {
-      delete d->retries.takeFirst();
+      d->retries.takeFirst().clear();
     }
   }
   else
@@ -1081,9 +1277,8 @@ void Smb4KMounter::timerEvent(QTimerEvent *)
   //
   if (d->checkTimeout >= 2500 && !isRunning() && d->importedShares.isEmpty())
   {
-    for (int i = 0; i < mountedSharesList().size(); ++i)
+    for (const SharePtr &share : mountedSharesList())
     {
-      Smb4KShare *share = mountedSharesList()[i];
       check(share);
       emit updated(share);
     }
@@ -1101,7 +1296,7 @@ void Smb4KMounter::timerEvent(QTimerEvent *)
 //
 // Linux arguments
 //
-bool Smb4KMounter::fillMountActionArgs(Smb4KShare *share, QVariantMap& map)
+bool Smb4KMounter::fillMountActionArgs(const SharePtr &share, QVariantMap& map)
 {
   // Find the mount program.
   const QString mount = findMountExecutable();
@@ -1118,7 +1313,7 @@ bool Smb4KMounter::fillMountActionArgs(Smb4KShare *share, QVariantMap& map)
 
   // Mount arguments.
   QMap<QString, QString> global_options = globalSambaOptions();
-  Smb4KCustomOptions *options  = Smb4KCustomOptionsManager::self()->findOptions(share);
+  OptionsPtr options = Smb4KCustomOptionsManager::self()->findOptions(share);
 
   if (options)
   {
@@ -1633,7 +1828,7 @@ bool Smb4KMounter::fillMountActionArgs(Smb4KShare *share, QVariantMap& map)
 //
 // FreeBSD and NetBSD arguments
 //
-bool Smb4KMounter::fillMountActionArgs(Smb4KShare *share, QVariantMap& map)
+bool Smb4KMounter::fillMountActionArgs(const SharePtr &share, QVariantMap& map)
 {
   // Find the mount program.
   const QString mount = findMountExecutable();
@@ -1650,7 +1845,7 @@ bool Smb4KMounter::fillMountActionArgs(Smb4KShare *share, QVariantMap& map)
   
   // Mount arguments.
   QMap<QString, QString> global_options = globalSambaOptions();
-  Smb4KCustomOptions *options  = Smb4KCustomOptionsManager::self()->findOptions(share);
+  OptionsPtr options  = Smb4KCustomOptionsManager::self()->findOptions(share);
 
   // Compile the list of arguments.
   QStringList args_list;
@@ -1787,7 +1982,7 @@ bool Smb4KMounter::fillMountActionArgs(Smb4KShare *share, QVariantMap& map)
 //
 // Dummy 
 //
-bool Smb4KMounter::fillMountActionArgs(Smb4KShare *, QVariantMap&)
+bool Smb4KMounter::fillMountActionArgs(const SharePtr &, QVariantMap&)
 {
   qWarning() << "Smb4KMounter::fillMountActionArgs() is not implemented!";
   qWarning() << "Mounting under this operating system is not supported...";
@@ -1800,7 +1995,7 @@ bool Smb4KMounter::fillMountActionArgs(Smb4KShare *, QVariantMap&)
 //
 // Linux arguments
 //
-bool Smb4KMounter::fillUnmountActionArgs(Smb4KShare *share, bool force, bool silent, QVariantMap &map)
+bool Smb4KMounter::fillUnmountActionArgs(const SharePtr &share, bool force, bool silent, QVariantMap &map)
 {
   //
   // The umount program
@@ -1853,7 +2048,7 @@ bool Smb4KMounter::fillUnmountActionArgs(Smb4KShare *share, bool force, bool sil
 //
 // FreeBSD and NetBSD arguments
 //
-bool Smb4KMounter::fillUnmountActionArgs(Smb4KShare *share, bool force, bool silent, QVariantMap &map)
+bool Smb4KMounter::fillUnmountActionArgs(const SharePtr &share, bool force, bool silent, QVariantMap &map)
 {
   //
   // The umount program
@@ -1906,7 +2101,7 @@ bool Smb4KMounter::fillUnmountActionArgs(Smb4KShare *share, bool force, bool sil
 //
 // Dummy
 //
-bool Smb4KMounter::fillUnmountActionArgs(Smb4KShare *, bool, bool, QVariantMap &)
+bool Smb4KMounter::fillUnmountActionArgs(const SharePtr &, bool, bool, QVariantMap &)
 {
   qWarning() << "Smb4KMounter::fillUnmountActionArgs() is not implemented!";
   qWarning() << "Unmounting under this operating system is not supported...";
@@ -1915,7 +2110,7 @@ bool Smb4KMounter::fillUnmountActionArgs(Smb4KShare *, bool, bool, QVariantMap &
 #endif
 
 
-void Smb4KMounter::check(Smb4KShare* share)
+void Smb4KMounter::check(const SharePtr &share)
 {
   // Get the info about the usage, etc.
   KDiskFreeSpaceInfo spaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(share->path());
@@ -1971,23 +2166,38 @@ void Smb4KMounter::check(Smb4KShare* share)
 void Smb4KMounter::slotStartJobs()
 {
   //
-  // Import the mounted shares
+  // Disconnect from Smb4KHardwareInterface.
   //
-  if (!d->firstImportDone)
-  {
-    import(true);
-  }
-  else
-  {
-    // Do nothing
-  }
+  disconnect(Smb4KHardwareInterface::self(), SIGNAL(networkConfigUpdated()), this, SLOT(slotStartJobs()));
   
   //
-  // Start the timer
-  //
-  if (d->timerId == -1)
+  // Start the import of shares
+  // 
+  if (Smb4KHardwareInterface::self()->isOnline())
   {
-    d->timerId = startTimer(TIMEOUT);
+    //
+    // Import the mounted shares
+    //
+    if (!d->firstImportDone)
+    {
+      import(true);
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    //
+    // Start the timer
+    //
+    if (d->timerId == -1)
+    {
+      d->timerId = startTimer(TIMEOUT);
+    }
+    else
+    {
+      // Do nothing
+    }
   }
   else
   {
@@ -1998,11 +2208,6 @@ void Smb4KMounter::slotStartJobs()
 
 void Smb4KMounter::slotAboutToQuit()
 {
-  //
-  // The application is about to quit. 
-  //
-  d->aboutToQuit = true;
-  
   //
   // Abort any actions
   //
@@ -2027,11 +2232,6 @@ void Smb4KMounter::slotAboutToQuit()
   if (Smb4KSettings::unmountSharesOnExit())
   {
     unmountAllShares(true, 0);
-    
-    while (isRunning())
-    {
-      QTest::qWait(TIMEOUT);
-    }
   }
   else
   {
@@ -2088,136 +2288,6 @@ void Smb4KMounter::slotAboutToQuit()
 }
 
 
-void Smb4KMounter::slotJobFinished(KJob* job)
-{
-  qDebug() << "Job finished";
-  
-  //
-  // The execute job
-  //
-  KAuth::ExecuteJob *execJob = static_cast<KAuth::ExecuteJob *>(job);
-  int errorCode = execJob->error();
-  Smb4KGlobal::Process process;
-  
-  if (execJob->objectName().startsWith(QLatin1String("MountJob")))
-  {
-    process = MountShare;
-  }
-  else
-  {
-    process = UnmountShare;
-  }
-  
-  //
-  // Process the returned data
-  //
-  if (errorCode == 0)
-  {
-    QVariantMap arguments = execJob->action().arguments();
-    QVariantMap returnedData = execJob->data();
-    QMapIterator<QString, QVariant> it(arguments);
-    
-    while (it.hasNext())
-    {
-      it.next();
-      QString index = it.key();
-      
-      // Get the data from the  action to create a share object that can be
-      // used subsequently to initiate a retry, to show notifications, etc.
-      Smb4KShare share;
-      share.setURL(it.value().toMap().value("mh_url").toUrl());
-      share.setWorkgroupName(it.value().toMap().value("mh_workgroup").toString());
-      share.setPath(it.value().toMap().value("mh_mountpoint").toString());
-      
-      QString errorMsg = returnedData.value(QString("mh_error_message_%1").arg(index)).toString();
-      
-      if (!errorMsg.isEmpty())
-      {
-        if (process == MountShare)
-        {
-#if defined(Q_OS_LINUX)
-          if (errorMsg.contains("mount error 13") || errorMsg.contains("mount error(13)") /* authentication error */)
-          {
-            if (Smb4KWalletManager::self()->showPasswordDialog(&share, 0))
-            {
-              d->retries << new Smb4KShare(share);
-            }
-            else
-            {
-              // Do nothing
-            }
-          }
-          else if (errorMsg.contains("Unable to find suitable address."))
-          {
-            // Swallow this
-          }
-          else
-          {
-            Smb4KNotification::mountingFailed(&share, errorMsg);
-          }
-#elif defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
-          if (errorMsg.contains("Authentication error") || errorMsg.contains("Permission denied"))
-          {
-            if (Smb4KWalletManager::self()->showPasswordDialog(&share, 0))
-            {
-              d->retries << new Smb4KShare(share);
-            }
-            else
-            {
-              // Do nothing
-            }
-          }
-          else
-          {
-            Smb4KNotification::mountingFailed(&share, errorMsg);
-          }
-#else
-          qWarning() << "Smb4KMounter::slotMountJobFinished(): Error handling not implemented!";
-          Smb4KNotification::mountingFailed(&share, errorMsg);
-#endif
-        }
-        else if (process == UnmountShare)
-        {
-          // No error handling needed, just report the error message.
-          Smb4KNotification::unmountingFailed(&share, errorMsg);
-        }
-        else
-        {
-          // Do nothing
-        }
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-  }
-  else
-  {
-    Smb4KNotification::actionFailed(errorCode);
-  }
-  
-  //
-  // Remove the job from the job list
-  //
-  removeSubjob(job);
-    
-  //
-  // Reset the busy cursor
-  //
-  if (!hasSubjobs() && modifyCursor())
-  {
-    QApplication::restoreOverrideCursor();
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  emit finished(process);
-}
-
-
 void Smb4KMounter::slotOnlineStateChanged(bool online)
 {
   if (online)
@@ -2239,7 +2309,7 @@ void Smb4KMounter::slotOnlineStateChanged(bool online)
     abortAll();
     
     // Mark all mounted shares as inaccessible
-    for (Smb4KShare *share : mountedSharesList())
+    for (const SharePtr &share : mountedSharesList())
     {
       share->setInaccessible(true);
     }
@@ -2270,7 +2340,7 @@ void Smb4KMounter::slotStatResult(KJob *job)
   //
   // Find the imported share
   //
-  Smb4KShare *importedShare = 0;
+  SharePtr importedShare;
   
   for (int i = 0; i < d->importedShares.size(); ++i)
   {
@@ -2333,45 +2403,6 @@ void Smb4KMounter::slotStatResult(KJob *job)
     importedShare->setForeign(true);
   }
   
-  //
-  // Check if we need to add an IP address or a workgroup/domain name
-  //
-  if (importedShare->workgroupName().isEmpty() || !importedShare->hasHostIP())
-  {
-    Smb4KHost *host = findHost(importedShare->hostName(), importedShare->workgroupName());
-
-    if (host)
-    {
-      // Set the IP address if necessary.
-      if (importedShare->hostIP().isEmpty() || QString::compare(host->ip(), importedShare->hostIP()) != 0)
-      {
-        importedShare->setHostIP(host->ip());
-      }
-      else
-      {
-        // Do nothing
-      }
-
-      // Set the workgroup/domain name if necessary.
-      if (importedShare->workgroupName().isEmpty())
-      {
-        importedShare->setWorkgroupName(host->workgroupName());
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-    else
-    {
-      // Do nothing
-    }
-  }
-  else
-  {
-    // Do nothing
-  }  
-  
   //
   // Search for a previously added mounted share and try to update it. If this fails,
   // add the share to the global list.
@@ -2380,7 +2411,7 @@ void Smb4KMounter::slotStatResult(KJob *job)
   {
     if (updateMountedShare(importedShare))
     {
-      Smb4KShare *updatedShare = findShareByPath(importedShare->path());
+      SharePtr updatedShare = findShareByPath(importedShare->path());
       
       if (updatedShare)
       {
@@ -2391,18 +2422,18 @@ void Smb4KMounter::slotStatResult(KJob *job)
         // Do nothing
       }
       
-      delete importedShare;
+      importedShare.clear();
     }
     else
     {
       if (addMountedShare(importedShare))
       {
         // Remove the share from the list of shares that are to be remounted
-        QMutableListIterator<Smb4KShare *> s(d->remounts);
+        QMutableListIterator<SharePtr> s(d->remounts);
 
         while (s.hasNext())
         {
-          Smb4KShare *remount = s.next();
+          SharePtr remount = s.next();
 
           if (!importedShare->isForeign() && QString::compare(remount->unc(), importedShare->unc(), Qt::CaseInsensitive) == 0)
           {
@@ -2421,7 +2452,7 @@ void Smb4KMounter::slotStatResult(KJob *job)
         d->newlyMounted += 1;
         emit mounted(importedShare);
         
-        if (d->importedShares.isEmpty())
+        if (d->importedShares.isEmpty() && !d->mountShares && !d->unmountShares)
         {
           if (d->firstImportDone)
           {
@@ -2450,13 +2481,13 @@ void Smb4KMounter::slotStatResult(KJob *job)
       }
       else
       {
-        delete importedShare;
+        importedShare.clear();
       }
     }
   }
   else
   {
-    delete importedShare;
+    importedShare.clear();
   }
   
   if (!d->firstImportDone && d->importedShares.isEmpty())
@@ -2470,39 +2501,44 @@ void Smb4KMounter::slotStatResult(KJob *job)
 }
 
 
+void Smb4KMounter::slotAboutToChangeProfile()
+{
+  //
+  // Save those shares that are to be remounted
+  //
+  if (Smb4KSettings::remountShares())
+  {
+    saveSharesForRemount();
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
 void Smb4KMounter::slotActiveProfileChanged(const QString &newProfile)
 {
-  if (QString::compare(d->activeProfile, newProfile) != 0)
+  if (d->activeProfile != newProfile)
   {
     // Stop the timer.
     killTimer(d->timerId);
 
-    // Check if the user wants to remount shares and save the
-    // shares for remount if so.
-    if (Smb4KSettings::remountShares())
-    {
-      saveSharesForRemount();
-    }
-    else
-    {
-      // Do nothing
-    }
-    
     abortAll();
     
     // Clear all remounts.
     while (!d->remounts.isEmpty())
     {
-      delete d->remounts.takeFirst();
+      d->remounts.takeFirst().clear();
     }
     
     // Clear all retries.
     while (!d->retries.isEmpty())
     {
-      delete d->retries.takeFirst();
+      d->retries.takeFirst().clear();
     }
     
-    // Unmount all shares and wait until done.
+    // Unmount all shares
     unmountAllShares(true, 0);
     
     // Reset some variables.
@@ -2511,7 +2547,7 @@ void Smb4KMounter::slotActiveProfileChanged(const QString &newProfile)
     d->firstImportDone = false;
     d->activeProfile = newProfile;
     
-    // Start the timer again.
+    // Restart the timer
     d->timerId = startTimer(TIMEOUT);
   }
   else
