@@ -3,7 +3,7 @@
     namespace.
                              -------------------
     begin                : So Jun 22 2014
-    copyright            : (C) 2014-2016 by Alexander Reinholdt
+    copyright            : (C) 2014-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -56,6 +56,6 @@ QUrl Smb4KNotificationActionRunner::mountpoint() const
 
 void Smb4KNotificationActionRunner::slotOpenShare()
 {
-  KRun::runUrl(m_mountpoint, "inode/directory", 0);
+  KRun::runUrl(m_mountpoint, "inode/directory", 0, KRun::RunFlags());
 }
 
