@@ -2,7 +2,7 @@
     This class manages the profiles that were defined by the user.
                              -------------------
     begin                : Mi Aug 06 2014
-    copyright            : (C) 2014-2016 by Alexander Reinholdt
+    copyright            : (C) 2014-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -86,14 +86,19 @@ Smb4KProfileManager* Smb4KProfileManager::self()
 
 void Smb4KProfileManager::setActiveProfile(const QString& name)
 {
-  bool changed = false;
+  //
+  // Check if the active profile is going to be changed. If so, 
+  // notify the program so that things can be done before the 
+  // profile is actually changed.
+  // 
+  bool change = false;
   
   if (d->useProfiles)
   {
-    if (QString::compare(name, d->activeProfile, Qt::CaseSensitive) != 0)
+    if (name != d->activeProfile)
     {
-      d->activeProfile = name;
-      changed = true;
+      emit aboutToChangeProfile();
+      change = true;
     }
     else
     {
@@ -104,17 +109,21 @@ void Smb4KProfileManager::setActiveProfile(const QString& name)
   {
     if (!d->activeProfile.isEmpty())
     {
-      d->activeProfile.clear();
-      changed = true;
+      emit aboutToChangeProfile();
+      change = true;
     }
     else
     {
       // Do nothing
     }
   }
-    
-  if (changed)
+  
+  //
+  // Now change the profile
+  // 
+  if (change)
   {
+    d->activeProfile = d->useProfiles ? name : QString();
     Smb4KSettings::setActiveProfile(d->activeProfile);
     emit activeProfileChanged(d->activeProfile);
   }
@@ -145,13 +154,13 @@ bool Smb4KProfileManager::useProfiles() const
 
 void Smb4KProfileManager::migrateProfile(const QString& from, const QString& to)
 {
-  QList< QPair<QString,QString> > list;
+  QList<QPair<QString,QString>> list;
   list << QPair<QString,QString>(from, to);
   migrateProfiles(list);
 }
 
 
-void Smb4KProfileManager::migrateProfiles(const QList< QPair<QString,QString> >& list)
+void Smb4KProfileManager::migrateProfiles(const QList<QPair<QString,QString>>& list)
 {
   if (d->useProfiles || (list.size() == 1 && list.first().second.isEmpty()))
   {
@@ -163,7 +172,6 @@ void Smb4KProfileManager::migrateProfiles(const QList< QPair<QString,QString> >&
       if (!to.isEmpty()) 
       {
         // Migrate one/the default profile to another one.
-        // 
         // First exchange the old profile.
         for (int j = 0; j < d->profiles.size(); ++j)
         {
@@ -310,21 +318,25 @@ void Smb4KProfileManager::removeProfiles(const QStringList& list, QWidget* paren
 
 void Smb4KProfileManager::slotConfigChanged()
 {
-  bool use_changed = false;
+  bool usageChanged = false;
   
-  // Use of profiles
+  //
+  // Check if the usage of profiles changed
+  //
   if (d->useProfiles != Smb4KSettings::useProfiles())
   {
     d->useProfiles = Smb4KSettings::useProfiles();
     emit profileUsageChanged(d->useProfiles);
-    use_changed = true;
+    usageChanged = true;
   }
   else
   {
     // Do nothing
   }
   
-  // List of profiles
+  //
+  // Updated the list of profiles
+  //
   if (d->profiles != Smb4KSettings::profilesList())
   {
     d->profiles = Smb4KSettings::profilesList();
@@ -334,9 +346,13 @@ void Smb4KProfileManager::slotConfigChanged()
   {
     // Do nothing
   }
-
-  // Migrate profile(s), if necessary.
-  if (use_changed && Smb4KSettings::useMigrationAssistant())
+  
+  //
+  // Migrate profiles.
+  // Profiles are only migrated, if the usage changed and the 
+  // user chose to use the migration assistant.
+  // 
+  if (usageChanged && Smb4KSettings::useMigrationAssistant())
   {
     QStringList from, to;
     
@@ -378,9 +394,10 @@ void Smb4KProfileManager::slotConfigChanged()
     // Do nothing
   }
   
-  // Active profile
-  if (!Smb4KSettings::activeProfile().isEmpty() &&
-      d->profiles.contains(Smb4KSettings::activeProfile()))
+  //
+  // Set the active profile
+  //
+  if (!Smb4KSettings::activeProfile().isEmpty() && d->profiles.contains(Smb4KSettings::activeProfile()))
   {
     setActiveProfile(Smb4KSettings::activeProfile());
   }
