@@ -2,7 +2,7 @@
     This class manages the profiles that were defined by the user.
                              -------------------
     begin                : Mi Aug 06 2014
-    copyright            : (C) 2014-2016 by Alexander Reinholdt
+    copyright            : (C) 2014-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -123,7 +123,7 @@ class Q_DECL_EXPORT Smb4KProfileManager : public QObject
      *                    is the "from" profile, the second one the 
      *                    "to" profile.
      */
-    void migrateProfiles(const QList< QPair<QString,QString> > &list);
+    void migrateProfiles(const QList<QPair<QString,QString>> &list);
     
     /**
      * Remove a profile with all of its entries.
@@ -160,6 +160,13 @@ class Q_DECL_EXPORT Smb4KProfileManager : public QObject
      */
     void removedProfile(const QString &profile);
     
+    /**
+     * This signal is emitted when the active profile is about 
+     * to be changed. You should connect to this signal, if you need 
+     * to save settings or the like to the OLD profile.
+     */
+    void aboutToChangeProfile();
+    
     /**
      * This signal is emitted when the active profile changed.
      * 
