@@ -2,7 +2,7 @@
     Smb4K's container class for information about a share.
                              -------------------
     begin                : Mo Jan 28 2008
-    copyright            : (C) 2008-2016 by Alexander Reinholdt
+    copyright            : (C) 2008-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -159,8 +159,16 @@ void Smb4KShare::setShareName(const QString &name)
 
 QString Smb4KShare::shareName() const
 {
-  QString share_name = d->url.path();
-  return share_name.remove('/');
+  if (d->url.path().startsWith('/'))
+  {
+    return d->url.path().remove(0, 1);
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  return d->url.path();
 }
 
 
@@ -422,7 +430,7 @@ void Smb4KShare::setInaccessible(bool in)
 
 bool Smb4KShare::isInaccessible() const
 {
-  return d->inaccessible;
+  return (d->mounted && d->inaccessible);
 }
 
 
@@ -435,7 +443,7 @@ void Smb4KShare::setForeign(bool foreign)
 
 bool Smb4KShare::isForeign() const
 {
-  return d->foreign;
+  return (d->mounted && d->foreign);
 }
 
 
@@ -1249,3 +1257,21 @@ void Smb4KShare::setShareIcon()
   }
 }
 
+
+void Smb4KShare::update(Smb4KShare* share)
+{
+  if (QString::compare(workgroupName(), share->workgroupName()) == 0 &&
+      (QString::compare(unc(), share->unc()) == 0 || QString::compare(homeUNC(), share->homeUNC()) == 0))
+  {
+    setMountData(share);
+    setURL(share->url());
+    setTypeString(share->typeString());
+    setComment(share->comment());
+    setHostIP(share->hostIP());
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
