@@ -40,6 +40,7 @@
 #include <KCoreAddons/Kdelibs4ConfigMigrator>
 #include <KCoreAddons/Kdelibs4Migration>
 #include <KDBusAddons/KDBusService>
+#include <KCrash/KCrash>
 
 using namespace Smb4KGlobal;
 
@@ -137,6 +138,11 @@ int main(int argc, char **argv)
     // Do nothing
   }
   
+  // Set attributes
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
+  QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, true);
+#endif
+  
   // Create the application
   QApplication app(argc, argv);
   
@@ -146,16 +152,14 @@ int main(int argc, char **argv)
   // Create the about data for Smb4K  
   KAboutData aboutData(QStringLiteral("smb4k"), i18n("Smb4K"), QStringLiteral(VERSION),
     i18n("Advanced network neighborhood browser and Samba share mounting utility"),
-    KAboutLicense::GPL_V2, i18n("\u00A9 2003-2017 Alexander Reinholdt"), QString(), 
+    KAboutLicense::GPL_V2, i18n("\u00A9 2003-2018 Alexander Reinholdt"), QString(), 
     QStringLiteral("http://smb4k.sourceforge.net"));
   
   // DBus prefix
   aboutData.setOrganizationDomain("kde.org");
   
   // Authors
-  aboutData.addAuthor(i18n("Alexander Reinholdt"),
-                      i18n("Developer"),
-                      QStringLiteral("alexander.reinholdt@kdemail.net"));
+  aboutData.addAuthor(i18n("Alexander Reinholdt"), i18n("Developer"), QStringLiteral("alexander.reinholdt@kdemail.net"));
   
   // Credits:
   // People who supported the Smb4K development by donating
@@ -193,6 +197,9 @@ int main(int argc, char **argv)
   // Unique application
   const KDBusService service(KDBusService::Unique);
   
+  // Use a crash handler
+  KCrash::setDrKonqiEnabled(true);
+  
   // Start the application
   return app.exec();
 }
