@@ -2,7 +2,7 @@
     The configuration dialog of Smb4K
                              -------------------
     begin                : Sa Apr 14 2007
-    copyright            : (C) 2004-2016 by Alexander Reinholdt
+    copyright            : (C) 2004-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -157,12 +157,14 @@ void Smb4KConfigDialog::setupDialog()
   profiles_area->setWidgetResizable(true);
   profiles_area->setFrameStyle(QFrame::NoFrame);
 
-  // Now add the pages to the configuration dialog
-  m_user_interface  = addPage(interface_area, Smb4KSettings::self(), i18n("User Interface"), "view-choose");
+  // 
+  // Pages to the configuration dialog
+  // 
+  m_user_interface  = addPage(interface_area, Smb4KSettings::self(), i18n("User Interface"), "preferences-desktop");
   m_network         = addPage(network_area, Smb4KSettings::self(), i18n("Network"), "network-workgroup");
   m_shares          = addPage(share_area, Smb4KSettings::self(), i18n("Shares"), "folder-network");
   m_authentication  = addPage(auth_area, Smb4KSettings::self(), i18n("Authentication"), "dialog-password");
-  m_samba           = addPage(samba_area, Smb4KSettings::self(), i18n("Samba"), "preferences-system-network");
+  m_samba           = addPage(samba_area, Smb4KSettings::self(), i18n("Samba"), "preferences-system-network-sharing");
 #if !defined(UNSUPPORTED_PLATFORM)
   m_mounting        = addPage(mount_area, Smb4KMountSettings::self(), i18n("Mounting"), "system-run");
 #endif
@@ -170,28 +172,22 @@ void Smb4KConfigDialog::setupDialog()
   m_custom_options  = addPage(custom_area, Smb4KSettings::self(), i18n("Custom Options"), "preferences-system-network");
   m_profiles        = addPage(profiles_area, Smb4KSettings::self(), i18n("Profiles"), "format-list-unordered");
 
+  // 
   // Connections
-  connect(custom_options, SIGNAL(customSettingsModified()),
-          this,           SLOT(slotEnableApplyButton()));
-  
-  connect(custom_options, SIGNAL(reloadCustomSettings()),
-          this,           SLOT(slotReloadCustomOptions()));
+  // 
+  connect(custom_options, SIGNAL(customSettingsModified()), this, SLOT(slotEnableApplyButton()));
+  connect(custom_options, SIGNAL(reloadCustomSettings()), this, SLOT(slotReloadCustomOptions()));
 
-  connect(auth_options,  SIGNAL(loadWalletEntries()),
-          this,          SLOT(slotLoadAuthenticationInformation()));
-           
-  connect(auth_options,  SIGNAL(saveWalletEntries()),
-          this,          SLOT(slotSaveAuthenticationInformation()));
-           
-  connect(auth_options,  SIGNAL(setDefaultLogin()),
-          this,          SLOT(slotSetDefaultLogin()));
-           
-  connect(auth_options,  SIGNAL(walletEntriesModified()),
-          this,          SLOT(slotEnableApplyButton()));
+  connect(auth_options, SIGNAL(loadWalletEntries()), this, SLOT(slotLoadAuthenticationInformation()));
+  connect(auth_options, SIGNAL(saveWalletEntries()), this, SLOT(slotSaveAuthenticationInformation()));
+  connect(auth_options, SIGNAL(setDefaultLogin()), this, SLOT(slotSetDefaultLogin()));
+  connect(auth_options, SIGNAL(walletEntriesModified()), this, SLOT(slotEnableApplyButton()));
   
-  connect(this,          SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)),
-          this,          SLOT(slotCheckPage(KPageWidgetItem*,KPageWidgetItem*)));
+  connect(this, SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)), this, SLOT(slotCheckPage(KPageWidgetItem*,KPageWidgetItem*)));
   
+  //
+  // Dialog size
+  //
   resize(QSize(800, 600));
 
   KConfigGroup group(Smb4KSettings::self()->config(), "ConfigDialog");
@@ -203,7 +199,7 @@ void Smb4KConfigDialog::loadCustomOptions()
 {
   if (m_custom_options)
   {
-    QList<Smb4KCustomOptions *> options = Smb4KCustomOptionsManager::self()->customOptions();
+    QList<OptionsPtr> options = Smb4KCustomOptionsManager::self()->customOptions();
     m_custom_options->widget()->findChild<Smb4KConfigPageCustomOptions *>()->insertCustomOptions(options);
   }
   else
@@ -217,7 +213,7 @@ void Smb4KConfigDialog::saveCustomOptions()
 {
   if (m_custom_options)
   {
-    QList<Smb4KCustomOptions *> options = m_custom_options->widget()->findChild<Smb4KConfigPageCustomOptions *>()->getCustomOptions();
+    QList<OptionsPtr> options = m_custom_options->widget()->findChild<Smb4KConfigPageCustomOptions *>()->getCustomOptions();
     Smb4KCustomOptionsManager::self()->replaceCustomOptions(options);
   }
   else
@@ -774,6 +770,13 @@ void Smb4KConfigDialog::updateWidgets()
 }
 
 
+void Smb4KConfigDialog::reject()
+{
+  Smb4KCustomOptionsManager::self()->resetCustomOptions();
+  QDialog::reject();
+}
+
+
 void Smb4KConfigDialog::slotLoadAuthenticationInformation()
 {
   Smb4KConfigPageAuthentication *auth_options = m_authentication->widget()->findChild<Smb4KConfigPageAuthentication *>();
@@ -906,23 +909,23 @@ void Smb4KConfigDialog::slotEnableApplyButton()
   
   if (!enable && custom_options && custom_options->customSettingsMaybeChanged())
   {
-    QList<Smb4KCustomOptions *> new_list = custom_options->getCustomOptions();
-    QList<Smb4KCustomOptions *> old_list = Smb4KCustomOptionsManager::self()->customOptions();
+    QList<OptionsPtr> new_list = custom_options->getCustomOptions();
+    QList<OptionsPtr> old_list = Smb4KCustomOptionsManager::self()->customOptions();
     
     if (new_list.size() == old_list.size())
     {
-      for (int i = 0; i < new_list.size(); ++i)
+      for (const OptionsPtr &no : new_list)
       {
-        for (int j = 0; j < old_list.size(); ++j)
+        for (const OptionsPtr &oo : old_list)
         {
-          if (!new_list[i]->equals(old_list.at(j)))
+          if (!no->equals(oo.data()))
           {
             enable = true;
             break;
           }
           else
           {
-            continue;
+            // Do nothing
           }
         }
         
@@ -932,7 +935,7 @@ void Smb4KConfigDialog::slotEnableApplyButton()
         }
         else
         {
-          continue;
+          // Do nothing
         }
       }
     }
