@@ -2,7 +2,7 @@
     The main window of Smb4K.
                              -------------------
     begin                : Di Jan 1 2008
-    copyright            : (C) 2008-2016 by Alexander Reinholdt
+    copyright            : (C) 2008-2018 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -103,29 +103,40 @@ Smb4KMainWindow::~Smb4KMainWindow()
 
 void Smb4KMainWindow::setupActions()
 {
+  // 
   // Quit action
+  // 
   QAction *quit_action = KStandardAction::quit(this, SLOT(slotQuit()), actionCollection());
   actionCollection()->addAction("quit_action", quit_action);
 
+  // 
   // Configure action
+  // 
   QAction *configure_action = KStandardAction::preferences(this, SLOT(slotConfigDialog()), actionCollection());
   actionCollection()->addAction("configure_action", configure_action);
 
+  // 
   // Dock widgets action menu
+  // 
   KActionMenu *dock_widgets_menu = new KActionMenu(KDE::icon("tab-duplicate"), i18n("Dock Widgets"), actionCollection());
   actionCollection()->addAction("dock_widgets_menu", dock_widgets_menu);
 
   m_dock_widgets = new QActionGroup(actionCollection());
   m_dock_widgets->setExclusive(false);
 
+  // 
   // Bookmarks menu and action
+  // 
   Smb4KBookmarkMenu *bookmarks = new Smb4KBookmarkMenu(Smb4KBookmarkMenu::MainWindow, this, this);
-  bookmarks->addBookmarkAction()->setEnabled(false);
+  QAction *addBookmarkAction = new QAction(KDE::icon("bookmark-new"), i18n("Add &Bookmark"), actionCollection());
+  addBookmarkAction->setEnabled(false);
   actionCollection()->addAction("bookmarks_menu", bookmarks);
-  actionCollection()->addAction("bookmark_action", bookmarks->addBookmarkAction());
-  connect(bookmarks->addBookmarkAction(), SIGNAL(triggered(bool)), SLOT(slotAddBookmark()));
+  actionCollection()->addAction("bookmark_action", addBookmarkAction);
+  connect(addBookmarkAction, SIGNAL(triggered(bool)), SLOT(slotAddBookmark()));
   
+  // 
   // Profiles menu
+  // 
   Smb4KProfilesMenu *profiles = new Smb4KProfilesMenu(this);
   actionCollection()->addAction("profiles_menu", profiles);
 }
@@ -166,11 +177,11 @@ void Smb4KMainWindow::setupStatusBar()
   connect(Smb4KWalletManager::self(), SIGNAL(initialized()),
           this, SLOT(slotWalletManagerInitialized()));
   
-  connect(Smb4KMounter::self(), SIGNAL(mounted(Smb4KShare*)),
-          this, SLOT(slotVisualMountFeedback(Smb4KShare*)));
+  connect(Smb4KMounter::self(), SIGNAL(mounted(SharePtr)),
+          this, SLOT(slotVisualMountFeedback(SharePtr)));
   
-  connect(Smb4KMounter::self(), SIGNAL(unmounted(Smb4KShare*)),
-          this, SLOT(slotVisualUnmountFeedback(Smb4KShare*)));
+  connect(Smb4KMounter::self(), SIGNAL(unmounted(SharePtr)),
+          this, SLOT(slotVisualUnmountFeedback(SharePtr)));
   
   connect(Smb4KMounter::self(), SIGNAL(aboutToStart(int)),
           this, SLOT(slotMounterAboutToStart(int)));
@@ -178,11 +189,11 @@ void Smb4KMainWindow::setupStatusBar()
   connect(Smb4KMounter::self(), SIGNAL(finished(int)),
           this, SLOT(slotMounterFinished(int)));
   
-  connect(Smb4KScanner::self(), SIGNAL(aboutToStart(Smb4KBasicNetworkItem*,int)),
-          this, SLOT(slotScannerAboutToStart(Smb4KBasicNetworkItem*,int)));
+  connect(Smb4KScanner::self(), SIGNAL(aboutToStart(NetworkItemPtr,int)),
+          this, SLOT(slotScannerAboutToStart(NetworkItemPtr,int)));
   
-  connect(Smb4KScanner::self(), SIGNAL(finished(Smb4KBasicNetworkItem*,int)),
-          this, SLOT(slotScannerFinished(Smb4KBasicNetworkItem*,int)));
+  connect(Smb4KScanner::self(), SIGNAL(finished(NetworkItemPtr,int)),
+          this, SLOT(slotScannerFinished(NetworkItemPtr,int)));
   
   connect(Smb4KSearch::self(), SIGNAL(aboutToStart(QString)),
           this, SLOT(slotSearchAboutToStart(QString)));
@@ -190,11 +201,11 @@ void Smb4KMainWindow::setupStatusBar()
   connect(Smb4KSearch::self(), SIGNAL(finished(QString)),
           this, SLOT(slotSearchFinished(QString)));
 
-  connect(Smb4KPrint::self(), SIGNAL(aboutToStart(Smb4KShare*)),
-          this, SLOT(slotPrintingAboutToStart(Smb4KShare*)));
+  connect(Smb4KPrint::self(), SIGNAL(aboutToStart(SharePtr)),
+          this, SLOT(slotPrintingAboutToStart(SharePtr)));
 
-  connect(Smb4KPrint::self(), SIGNAL(finished(Smb4KShare*)),
-          this, SLOT(slotPrintingFinished(Smb4KShare*)));
+  connect(Smb4KPrint::self(), SIGNAL(finished(SharePtr)),
+          this, SLOT(slotPrintingFinished(SharePtr)));
 
   connect(Smb4KSynchronizer::self(), SIGNAL(aboutToStart(QString)),
           this, SLOT(slotSynchronizerAboutToStart(QString)));
@@ -202,19 +213,16 @@ void Smb4KMainWindow::setupStatusBar()
   connect(Smb4KSynchronizer::self(), SIGNAL(finished(QString)),
           this, SLOT(slotSynchronizerFinished(QString)));
 
-  connect(Smb4KPreviewer::self(), SIGNAL(aboutToStart(Smb4KShare*,QUrl)),
-          this, SLOT(slotPreviewerAboutToStart(Smb4KShare*,QUrl)));
+  connect(Smb4KPreviewer::self(), SIGNAL(aboutToStart(SharePtr,QUrl)),
+          this, SLOT(slotPreviewerAboutToStart(SharePtr,QUrl)));
 
-  connect(Smb4KPreviewer::self(), SIGNAL(finished(Smb4KShare*,QUrl)),
-          this, SLOT(slotPreviewerFinished(Smb4KShare*,QUrl)));
+  connect(Smb4KPreviewer::self(), SIGNAL(finished(SharePtr,QUrl)),
+          this, SLOT(slotPreviewerFinished(SharePtr,QUrl)));
 }
 
 
 void Smb4KMainWindow::setupView()
 {
-  // There is no active part initially. Set it 0.
-  m_active_part = 0;
-
   // We do not set a central widget, because it causes "problems"
   // with the dock widgets. We have the nested dock widget property
   // set to true, so we can arrange the dock widgets as we like,
@@ -643,10 +651,10 @@ void Smb4KMainWindow::slotSettingsChanged(const QString &)
 
 void Smb4KMainWindow::slotAddBookmark()
 {
-  if (m_active_part)
+  if (m_manager->activePart())
   {
     Smb4KEvent *customEvent = new Smb4KEvent(Smb4KEvent::AddBookmark);
-    QApplication::postEvent(m_active_part, customEvent);
+    QApplication::postEvent(m_manager->activePart(), customEvent);
   }
   else
   {
@@ -681,7 +689,7 @@ void Smb4KMainWindow::slotWalletManagerInitialized()
 }
 
 
-void Smb4KMainWindow::slotScannerAboutToStart(Smb4KBasicNetworkItem *item, int process)
+void Smb4KMainWindow::slotScannerAboutToStart(const NetworkItemPtr &item, int process)
 {
   Q_ASSERT(item);
 
@@ -694,13 +702,13 @@ void Smb4KMainWindow::slotScannerAboutToStart(Smb4KBasicNetworkItem *item, int p
     }
     case LookupDomainMembers:
     {
-      Smb4KWorkgroup *workgroup = static_cast<Smb4KWorkgroup *>(item);
+      WorkgroupPtr workgroup = item.staticCast<Smb4KWorkgroup>();
       statusBar()->showMessage(i18n("Looking for hosts in domain %1...", workgroup->workgroupName()), 0);
       break;
     }
     case LookupShares:
     {
-      Smb4KHost *host = static_cast<Smb4KHost *>(item);
+      HostPtr host = item.staticCast<Smb4KHost>();
       statusBar()->showMessage(i18n("Looking for shares provided by host %1...", host->hostName()), 0);
       break;
     }
@@ -726,7 +734,7 @@ void Smb4KMainWindow::slotScannerAboutToStart(Smb4KBasicNetworkItem *item, int p
 }
 
 
-void Smb4KMainWindow::slotScannerFinished(Smb4KBasicNetworkItem */*item*/, int /*process*/)
+void Smb4KMainWindow::slotScannerFinished(const NetworkItemPtr &/*item*/, int /*process*/)
 {
   if (!coreIsRunning())
   {
@@ -794,7 +802,7 @@ void Smb4KMainWindow::slotMounterFinished(int /*process*/)
 }
 
 
-void Smb4KMainWindow::slotVisualMountFeedback(Smb4KShare *share)
+void Smb4KMainWindow::slotVisualMountFeedback(const SharePtr &share)
 {
   Q_ASSERT(share);
   
@@ -848,7 +856,7 @@ void Smb4KMainWindow::slotVisualMountFeedback(Smb4KShare *share)
 }
 
 
-void Smb4KMainWindow::slotVisualUnmountFeedback(Smb4KShare *share)
+void Smb4KMainWindow::slotVisualUnmountFeedback(const SharePtr &share)
 {
   Q_ASSERT(share);
   
@@ -971,7 +979,7 @@ void Smb4KMainWindow::slotSearchFinished(const QString &/*string*/)
 }
 
 
-void Smb4KMainWindow::slotPrintingAboutToStart(Smb4KShare *printer)
+void Smb4KMainWindow::slotPrintingAboutToStart(const SharePtr &printer)
 {
   statusBar()->showMessage(i18n("Sending file to printer %1...", printer->unc()), 0);
 
@@ -986,7 +994,7 @@ void Smb4KMainWindow::slotPrintingAboutToStart(Smb4KShare *printer)
 }
 
 
-void Smb4KMainWindow::slotPrintingFinished(Smb4KShare */*printer*/)
+void Smb4KMainWindow::slotPrintingFinished(const SharePtr &/*printer*/)
 {
   if (!coreIsRunning())
   {
@@ -1031,7 +1039,7 @@ void Smb4KMainWindow::slotSynchronizerFinished(const QString &/*dest*/)
 }
 
 
-void Smb4KMainWindow::slotPreviewerAboutToStart(Smb4KShare *share, const QUrl &/*url*/)
+void Smb4KMainWindow::slotPreviewerAboutToStart(const SharePtr &share, const QUrl &/*url*/)
 {
   Q_ASSERT(share);
 
@@ -1048,7 +1056,7 @@ void Smb4KMainWindow::slotPreviewerAboutToStart(Smb4KShare *share, const QUrl &/
 }
 
 
-void Smb4KMainWindow::slotPreviewerFinished(Smb4KShare */*share*/, const QUrl &/*url*/)
+void Smb4KMainWindow::slotPreviewerFinished(const SharePtr &/*share*/, const QUrl &/*url*/)
 {
   if (!coreIsRunning())
   {
@@ -1065,81 +1073,134 @@ void Smb4KMainWindow::slotPreviewerFinished(Smb4KShare */*share*/, const QUrl &/
 
 void Smb4KMainWindow::slotActivePartChanged(KParts::Part *part)
 {
-  Q_ASSERT(part);
+  if (part)
+  {    
+    //
+    // Remove all connections to Smb4KMainWindow::slotEnableBookmarkAction() and
+    // disable the bookmark action.
+    //
+    disconnect(this, SLOT(slotEnableBookmarkAction()));
+    actionCollection()->action("bookmark_action")->setEnabled(false);
 
-  // First break the connections and disable the actions
-  if (m_active_part)
-  {
-    // Bookmark action
-    QAction *bookmark_action = m_active_part->actionCollection()->action("bookmark_action");
+    Smb4KBookmarkMenu *bookmarkMenu = findChild<Smb4KBookmarkMenu *>();
 
-    if (bookmark_action)
+    if (bookmarkMenu)
     {
-      disconnect(bookmark_action, SIGNAL(changed()), this, SLOT(slotEnableBookmarkAction()));
-      actionCollection()->action("bookmark_action")->setEnabled(false);
+      Smb4KEvent *customEvent = new Smb4KEvent(Smb4KEvent::DisableBookmarkAction);
+      QApplication::postEvent(bookmarkMenu, customEvent);
     }
     else
     {
       // Do nothing
     }
+    
+    // 
+    // Prepare the dynamic action list for the main window
+    //
+    QList<QAction *> dynamicList;
+    
+    for (QAction *action : m_manager->activePart()->actionCollection()->actions())
+    {
+      if (action)
+      {
+        if (QString::compare(action->objectName(), "bookmark_action") == 0)
+        {
+          actionCollection()->action("bookmark_action")->setEnabled(action->isEnabled());
+          
+          if (bookmarkMenu)
+          {
+            Smb4KEvent *customEvent = 0;
+            
+            if (action->isEnabled())
+            {
+              customEvent = new Smb4KEvent(Smb4KEvent::EnableBookmarkAction);
+            }
+            else
+            {
+              customEvent = new Smb4KEvent(Smb4KEvent::DisableBookmarkAction);
+            }
+              
+            QApplication::postEvent(bookmarkMenu, customEvent);
+          }
+          else
+          {
+            // Do nothing
+          }          
+          
+          connect(action, SIGNAL(changed()), this, SLOT(slotEnableBookmarkAction()));
+          continue;
+        }
+        else if (QString::compare(action->objectName(), "filemanager_action") == 0)
+        {
+          continue;
+        }
+        else if (QString::compare(action->objectName(), "konsole_action") == 0)
+        {
+          continue;
+        }
+        else if (QString::compare(action->objectName(), "icon_view_action") == 0)
+        {
+          continue;
+        }
+        else if (QString::compare(action->objectName(), "list_view_action") == 0)
+        {
+          continue;
+        }
+        else
+        {
+          // Do nothing
+        }
+
+        dynamicList << action;
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+
+    //
+    // Remove old and insert new dynamic action list
+    // 
+    unplugActionList("dynamic_list");
+    plugActionList("dynamic_list", dynamicList);
   }
   else
   {
     // Do nothing
   }
-
-  // Let m_active_part point to the new active part.
-  m_active_part = part;
-  
-  // Setup actions
-  QList<QAction *> dynamic_list;
-
-  for (int i = 0; i < m_active_part->actionCollection()->actions().size(); ++i)
-  {
-    QAction *action = m_active_part->actionCollection()->action(i);
-
-    if (QString::compare(action->objectName(), "bookmark_action") == 0)
-    {
-      actionCollection()->action("bookmark_action")->setEnabled(action->isEnabled());
-      connect(action, SIGNAL(changed()), this, SLOT(slotEnableBookmarkAction()));
-      continue;
-    }
-    else if (QString::compare(action->objectName(), "filemanager_action") == 0)
-    {
-      continue;
-    }
-    else if (QString::compare(action->objectName(), "konsole_action") == 0)
-    {
-      continue;
-    }
-    else if (QString::compare(action->objectName(), "icon_view_action") == 0)
-    {
-      continue;
-    }
-    else if (QString::compare(action->objectName(), "list_view_action") == 0)
-    {
-      continue;
-    }
-    else
-    {
-      // Do nothing
-    }
-
-    dynamic_list << action;
-  }
-
-  unplugActionList("dynamic_list");
-  plugActionList("dynamic_list", dynamic_list);
 }
 
 
 void Smb4KMainWindow::slotEnableBookmarkAction()
 {
-  QAction *action = m_active_part->actionCollection()->action("bookmark_action");
+  QAction *action = m_manager->activePart()->actionCollection()->action("bookmark_action");
 
   if (action)
   {
     actionCollection()->action("bookmark_action")->setEnabled(action->isEnabled());
+    
+    Smb4KBookmarkMenu *bookmarkMenu = findChild<Smb4KBookmarkMenu *>();
+    
+    if (bookmarkMenu)
+    {
+      Smb4KEvent *customEvent = 0;
+      
+      if (action->isEnabled())
+      {
+        customEvent = new Smb4KEvent(Smb4KEvent::EnableBookmarkAction);
+      }
+      else
+      {
+        customEvent = new Smb4KEvent(Smb4KEvent::DisableBookmarkAction);
+      }
+        
+      QApplication::postEvent(bookmarkMenu, customEvent);
+    }
+    else
+    {
+      // Do nothing
+    }
   }
   else
   {
@@ -1154,7 +1215,7 @@ void Smb4KMainWindow::slotNetworkBrowserVisibilityChanged(bool visible)
   
   if (dock)
   {
-    if (visible && m_browser_part != m_active_part)
+    if (visible && m_manager->activePart() != m_browser_part)
     {
       m_manager->setActivePart(m_browser_part);
     }
@@ -1176,7 +1237,7 @@ void Smb4KMainWindow::slotSharesViewVisibilityChanged(bool visible)
   
   if (dock)
   {
-    if (visible && m_shares_part != m_active_part)
+    if (visible && m_manager->activePart() != m_shares_part)
     {
       m_manager->setActivePart(m_shares_part);
     }
@@ -1198,7 +1259,7 @@ void Smb4KMainWindow::slotSearchDialogVisibilityChanged(bool visible)
   
   if (dock)
   {
-    if (visible && m_search_part != m_active_part)
+    if (visible && m_manager->activePart() != m_search_part)
     {
       m_manager->setActivePart(m_search_part);
     }
