@@ -3,7 +3,7 @@
     browser of Smb4K.
                              -------------------
     begin                : Fr Jan 5 2007
-    copyright            : (C) 2007-2016 by Alexander Reinholdt
+    copyright            : (C) 2007-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -45,6 +45,7 @@
 #include "core/smb4kbookmarkhandler.h"
 #include "core/smb4kcustomoptionsmanager.h"
 #include "core/smb4kcustomoptions.h"
+#include "core/smb4kworkgroup.h"
 
 // Qt includes
 #include <QEvent>
@@ -137,26 +138,26 @@ Smb4KNetworkBrowserPart::Smb4KNetworkBrowserPart(QWidget *parentWidget, QObject
   connect(m_widget, SIGNAL(itemActivated(QTreeWidgetItem*,int)),
           this, SLOT(slotItemActivated(QTreeWidgetItem*,int)));
   
-  connect(Smb4KScanner::self(), SIGNAL(workgroups(QList<Smb4KWorkgroup*>)),
-          this, SLOT(slotWorkgroups(QList<Smb4KWorkgroup*>)));
+  connect(Smb4KScanner::self(), SIGNAL(workgroups()),
+          this, SLOT(slotWorkgroups()));
   
-  connect(Smb4KScanner::self(), SIGNAL(hosts(Smb4KWorkgroup*,QList<Smb4KHost*>)),
-          this, SLOT(slotWorkgroupMembers(Smb4KWorkgroup*,QList<Smb4KHost*>)));
+  connect(Smb4KScanner::self(), SIGNAL(hosts(WorkgroupPtr)),
+          this, SLOT(slotWorkgroupMembers(WorkgroupPtr)));
   
-  connect(Smb4KScanner::self(), SIGNAL(shares(Smb4KHost*,QList<Smb4KShare*>)),
-          this, SLOT(slotShares(Smb4KHost*,QList<Smb4KShare*>)));
+  connect(Smb4KScanner::self(), SIGNAL(shares(HostPtr)),
+          this, SLOT(slotShares(HostPtr)));
   
-  connect(Smb4KScanner::self(), SIGNAL(authError(Smb4KHost*,int)),
-          this, SLOT(slotAuthError(Smb4KHost*,int)));
+  connect(Smb4KScanner::self(), SIGNAL(authError(HostPtr,int)),
+          this, SLOT(slotAuthError(HostPtr,int)));
   
-  connect(Smb4KScanner::self(), SIGNAL(aboutToStart(Smb4KBasicNetworkItem*,int)),
-          this, SLOT(slotScannerAboutToStart(Smb4KBasicNetworkItem*,int)));
+  connect(Smb4KScanner::self(), SIGNAL(aboutToStart(NetworkItemPtr,int)),
+          this, SLOT(slotScannerAboutToStart(NetworkItemPtr,int)));
   
-  connect(Smb4KScanner::self(), SIGNAL(finished(Smb4KBasicNetworkItem*,int)),
-          this, SLOT(slotScannerFinished(Smb4KBasicNetworkItem*,int)));
+  connect(Smb4KScanner::self(), SIGNAL(finished(NetworkItemPtr,int)),
+          this, SLOT(slotScannerFinished(NetworkItemPtr,int)));
   
-  connect(Smb4KScanner::self(), SIGNAL(ipAddress(Smb4KHost*)),
-          this, SLOT(slotAddIPAddress(Smb4KHost*)));
+  connect(Smb4KScanner::self(), SIGNAL(ipAddress(HostPtr)),
+          this, SLOT(slotAddIPAddress(HostPtr)));
   
   connect(Smb4KMounter::self(), SIGNAL(aboutToStart(int)),
           this, SLOT(slotMounterAboutToStart(int)));
@@ -164,11 +165,11 @@ Smb4KNetworkBrowserPart::Smb4KNetworkBrowserPart(QWidget *parentWidget, QObject
   connect(Smb4KMounter::self(), SIGNAL(finished(int)),
           this, SLOT(slotMounterFinished(int)));
   
-  connect(Smb4KMounter::self(), SIGNAL(mounted(Smb4KShare*)),
-          this, SLOT(slotShareMounted(Smb4KShare*)));
+  connect(Smb4KMounter::self(), SIGNAL(mounted(SharePtr)),
+          this, SLOT(slotShareMounted(SharePtr)));
   
-  connect(Smb4KMounter::self(), SIGNAL(unmounted(Smb4KShare*)),
-          this, SLOT(slotShareUnmounted(Smb4KShare*)));
+  connect(Smb4KMounter::self(), SIGNAL(unmounted(SharePtr)),
+          this, SLOT(slotShareUnmounted(SharePtr)));
   
   connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
           this, SLOT(slotAboutToQuit()));
@@ -213,8 +214,8 @@ void Smb4KNetworkBrowserPart::setupActions()
   connect(print_action, SIGNAL(triggered(bool)), this, SLOT(slotPrint(bool)));
 
   KDualAction *mount_action = new KDualAction(this);
-  KGuiItem mount_item(i18n("&Mount"), KDE::icon("emblem-mounted"));
-  KGuiItem unmount_item(i18n("&Unmount"), KDE::icon("emblem-unmounted"));
+  KGuiItem mount_item(i18n("&Mount"), KDE::icon("media-mount"));
+  KGuiItem unmount_item(i18n("&Unmount"), KDE::icon("media-eject"));
   mount_action->setActiveGuiItem(mount_item);
   mount_action->setInactiveGuiItem(unmount_item);
   mount_action->setActive(true);
@@ -694,555 +695,187 @@ void Smb4KNetworkBrowserPart::slotItemActivated(QTreeWidgetItem *item, int /*col
 }
 
 
-void Smb4KNetworkBrowserPart::slotWorkgroups(const QList<Smb4KWorkgroup *> &list)
+void Smb4KNetworkBrowserPart::slotWorkgroups()
 {
-  // Process the list.
-  if (!list.isEmpty())
+  //
+  // Process the global workgroup list
+  //
+  if (!workgroupsList().isEmpty())
   {
-    // Remove obsolete workgroup items from the tree widget.
-    for (int i = 0; i < m_widget->topLevelItemCount(); ++i)
+    //
+    // Remove obsolete workgroups and update existing ones
+    //
+    QTreeWidgetItemIterator itemIt(m_widget, QTreeWidgetItemIterator::All);
+    
+    while (*itemIt)
     {
-      Smb4KNetworkBrowserItem *workgroup_item = static_cast<Smb4KNetworkBrowserItem *>(m_widget->topLevelItem(i));
-
-      for (int j = 0; j < list.size(); ++j)
+      Smb4KNetworkBrowserItem *networkItem = static_cast<Smb4KNetworkBrowserItem *>(*itemIt);
+      
+      if (networkItem->type() == Workgroup)
       {
-        bool found_workgroup = false;
-
-        if (QString::compare(list.at(j)->workgroupName(), workgroup_item->workgroupItem()->workgroupName()) == 0)
+        WorkgroupPtr workgroup = findWorkgroup(networkItem->workgroupItem()->workgroupName());
+        
+        if (workgroup)
         {
-          // Check if the master browser is still the same.
-          if (QString::compare(list.at(j)->masterBrowserName(), workgroup_item->workgroupItem()->masterBrowserName()) != 0)
+          networkItem->update();
+          
+          // Update the master browser
+          for (int i = 0; i < networkItem->childCount(); ++i)
           {
-            // We found the workgroup.
-            bool found_new_master_browser = false;
-
-            // Find the old and the new master browser.
-            for (int k = 0; k < workgroup_item->childCount(); ++k)
-            {
-              Smb4KNetworkBrowserItem *host_item = static_cast<Smb4KNetworkBrowserItem *>(workgroup_item->child(k));
-
-              if (QString::compare(workgroup_item->workgroupItem()->masterBrowserName(), host_item->hostItem()->hostName()) == 0)
-              {
-                // This is the old master browser. Update it.
-                Smb4KHost *host = findHost(host_item->hostItem()->hostName(), host_item->hostItem()->workgroupName());
-
-                if (host)
-                {
-                  // Update.
-                  host_item->update(host);
-                }
-                else
-                {
-                  // The old master is not available anymore.
-                  // Remove it.
-                  delete host_item;
-                }
-
-                continue;
-              }
-              else if (QString::compare(list.at(j)->masterBrowserName(), host_item->hostItem()->hostName()) == 0)
-              {
-                // This is the new master browser. Update it.
-                Smb4KHost *host = findHost(host_item->hostItem()->hostName(), host_item->hostItem()->workgroupName());
-
-                if (host)
-                {
-                  // Update.
-                  host_item->update(host);
-                }
-                else
-                {
-                  // Huh???
-                }
-
-                continue;
-              }
-              else
-              {
-                continue;
-              }
-            }
-
-            if (!found_new_master_browser)
-            {
-              // The new master browser is not in the tree widget, so we have to put it there.
-              Smb4KHost *host = findHost(workgroup_item->workgroupItem()->masterBrowserName(), workgroup_item->workgroupItem()->workgroupName());
-              (void) new Smb4KNetworkBrowserItem(workgroup_item, host);
-            }
-            else
-            {
-              // Do nothing
-            }
+            Smb4KNetworkBrowserItem *host = static_cast<Smb4KNetworkBrowserItem *>(networkItem->child(i));
+            host->update();
           }
-          else
-          {
-            // Do nothing
-          }
-
-          // Update the workgroup item.
-          workgroup_item->update(list.at(j));
-
-          // We found the workgroup.
-          found_workgroup = true;
-
-          break;
         }
         else
         {
-          continue;
+          delete networkItem;
         }
-
-        // Remove the workgroup from the tree widget if it is obsolete.
-        if (!found_workgroup)
-        {
-          delete workgroup_item;
-        }
-        else
-        {
-          // Do nothing
-        }
-      }
-    }
-
-    // Put the new workgroup items into the tree widget.
-    for (int i = 0; i < list.size(); ++i)
-    {
-      QList<QTreeWidgetItem *> items = m_widget->findItems(list.at(i)->workgroupName(), Qt::MatchFixedString, Smb4KNetworkBrowser::Network);
-
-      if (items.isEmpty())
-      {
-        // The workgroup is not in the tree widget. Add it.
-        (void) new Smb4KNetworkBrowserItem(m_widget, list.at(i));
-        continue;
       }
       else
       {
-        continue;
+        // Do nothing
+      }
+        
+      ++itemIt;
+    }
+    
+    //
+    // Add new workgroups to the tree widget
+    //
+    for (const WorkgroupPtr &workgroup : workgroupsList())
+    {
+      QList<QTreeWidgetItem *> items = m_widget->findItems(workgroup->workgroupName(), Qt::MatchFixedString, Smb4KNetworkBrowser::Network);
+      
+      if (items.isEmpty())
+      {
+        (void) new Smb4KNetworkBrowserItem(m_widget, workgroup);
+      }
+      else
+      {
+        // Do nothing
       }
     }
-
-    // Sort the items.
+   
+    //
+    // Sort the items
+    //
     m_widget->sortItems(Smb4KNetworkBrowser::Network, Qt::AscendingOrder);
   }
   else
   {
-    // Clear the tree widget.
+    //
+    // Clear the tree widget
+    //
     m_widget->clear();
   }
 }
 
 
-void Smb4KNetworkBrowserPart::slotWorkgroupMembers(Smb4KWorkgroup *workgroup, const QList<Smb4KHost *> &list)
+void Smb4KNetworkBrowserPart::slotWorkgroupMembers(const WorkgroupPtr &workgroup)
 {
   if (workgroup)
   {
-    // Find the workgroup.
+    //
+    // Find the workgroup(s) 
+    //
     QList<QTreeWidgetItem *> workgroups = m_widget->findItems(workgroup->workgroupName(), Qt::MatchFixedString, Smb4KNetworkBrowser::Network);
-
-    if (!list.isEmpty())
-    {
-      QMutableListIterator<QTreeWidgetItem *> it(workgroups);
-      
-      while (it.hasNext())
-      {
-        Smb4KNetworkBrowserItem *workgroup_item = static_cast<Smb4KNetworkBrowserItem *>(it.next());
-        
-        // Remove obsolete hosts from the workgroup.
-        for (int j = 0; j < workgroup_item->childCount(); ++j)
-        {
-          Smb4KNetworkBrowserItem *host_item = static_cast<Smb4KNetworkBrowserItem *>(workgroup_item->child(j));
-          bool found_host = false;
-
-          for (int k = 0; k < list.size(); ++k)
-          {
-            if (QString::compare(list.at(k)->workgroupName(), host_item->hostItem()->workgroupName()) == 0 &&
-                 QString::compare(list.at(k)->hostName(), host_item->hostItem()->hostName()) == 0)
-            {
-              found_host = true;
-              break;
-            }
-            else
-            {
-              continue;
-            }
-          }
-
-          if (!found_host)
-          {
-            delete host_item;
-          }
-          else
-          {
-            // Do nothing
-          }
-        }
-        
-        // Add new hosts to the workgroup and update the existing ones.
-        for (int j = 0; j < list.size(); ++j)
-        {
-          if (QString::compare(list.at(j)->workgroupName(), workgroup_item->workgroupItem()->workgroupName()) == 0)
-          {
-            bool found_host = false;
-
-            for (int k = 0; k < workgroup_item->childCount(); ++k)
-            {
-              Smb4KNetworkBrowserItem *host_item = static_cast<Smb4KNetworkBrowserItem *>(workgroup_item->child(k));
-
-              if (QString::compare(list.at(j)->hostName(), host_item->hostItem()->hostName()) == 0)
-              {
-                host_item->update(list.at(j));
-                found_host = true;
-                break;
-              }
-              else
-              {
-                continue;
-              }
-            }
-
-            if (!found_host)
-            {
-              (void) new Smb4KNetworkBrowserItem(workgroup_item, list.at(j));
-            }
-            else
-            {
-              // Do nothing
-            }
-
-            continue;
-          }
-          else
-          {
-            continue;
-          }
-        }
-        
-        // If the workgroup item has children, expand it if necessary. 
-        // Otherwise there is no use in keeping it, so remove it.
-        if (workgroup_item->childCount() != 0)
-        {
-          if (Smb4KSettings::autoExpandNetworkItems() && !workgroup_item->isExpanded())
-          {
-            m_widget->expandItem(workgroup_item);
-          }
-          else
-          {
-            // Do nothing
-          }
-        }
-        else
-        {
-          delete workgroup_item;
-        }
-
-        // Sort the items.
-        m_widget->sortItems(Smb4KNetworkBrowser::Network, Qt::AscendingOrder);     
-      }
-    }
-    else
-    {
-      // The workgroup(s) has/have no children (anymore). Remove
-      // it/them.
-      while (!workgroups.isEmpty())
-      {
-        delete workgroups.takeFirst();
-      }
-    }
-  }
-  else
-  {
-    // This is the list of all hosts available on the network.
-    if (!list.isEmpty())
-    {
-      // Loop through the network neighborhood tree. Remove the obsolete host
-      // and update the still available ones.
-      QTreeWidgetItemIterator it(m_widget);
-      
-      while (*it)
-      {
-        Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>(*it);
-        
-        switch (item->type())
-        {
-          case Host:
-          {
-            bool found = false;
-            
-            for (int i = 0; i < list.size(); ++i)
-            {
-              if (QString::compare(list.at(i)->workgroupName(), item->hostItem()->workgroupName(), Qt::CaseInsensitive) == 0 &&
-                  QString::compare(list.at(i)->unc(), item->hostItem()->unc(), Qt::CaseInsensitive) == 0)
-              {
-                item->update(list.at(i));
-                found = true;
-                break;
-              }
-              else
-              {
-                continue;
-              }
-            }
-            
-            if (!found)
-            {
-              delete item;
-            }
-            else
-            {
-              // Do nothing
-            }            
-            break;
-          }
-          default:
-          {
-            break;
-          }
-        }
-        
-        ++it;
-      }
-      
-      // Now add the new hosts. For this we search the network neighborhood
-      // tree and add the host if either there is no match or the found host(s)
-      // belong(s) to another workgroup.
-      for (int i = 0; i < list.size(); ++i)
-      {
-        QList<QTreeWidgetItem *> hosts = m_widget->findItems(list.at(i)->hostName(), 
-                                                              Qt::MatchFixedString|Qt::MatchRecursive, 
-                                                              Smb4KNetworkBrowser::Network);
-        
-        if (!hosts.isEmpty())
-        {
-          bool match = false;
-          
-          for (int j = 0; j < hosts.size(); ++j)
-          {
-            Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>(hosts.at(i));
-            
-            if (item->type() == Host &&
-                 QString::compare(list.at(i)->workgroupName(), item->hostItem()->workgroupName(), Qt::CaseInsensitive) == 0 &&
-                 QString::compare(list.at(i)->unc(), item->hostItem()->unc(), Qt::CaseInsensitive) == 0)
-            {
-              match = true;
-              break;
-            }
-            else
-            {
-              continue;
-            }
-          }
-          
-          if (!match)
-          {
-            // This host is new. Add it to the list widget and create the
-            // workgroup item as well if needed.
-            QList<QTreeWidgetItem *> workgroups = m_widget->findItems(list.at(i)->workgroupName(), Qt::MatchFixedString, Smb4KNetworkBrowser::Network);
-            
-            if (!workgroups.isEmpty())
-            {
-              for (int j = 0; j < workgroups.size(); ++j)
-              {
-                Smb4KNetworkBrowserItem *workgroup_item = static_cast<Smb4KNetworkBrowserItem *>(workgroups.at(j));
-
-                if (workgroup_item)
-                {
-                  // FIXME: Do we need to change the ·ðmaster browser here?
-                  (void) new Smb4KNetworkBrowserItem(workgroup_item, list.at(i));
-                  
-                  if (Smb4KSettings::autoExpandNetworkItems() && !workgroup_item->isExpanded())
-                  {
-                    m_widget->expandItem(workgroup_item);
-                  }
-                  else
-                  {
-                    // Do nothing
-                  }
-                  continue;
-                }
-                else
-                {
-                  continue;
-                }
-              }
-            }
-            else
-            {
-              // We need to create the workgroup and the host item. Since the workgroup
-              // was not found in the browser, we can assume, that it is also not available
-              // in the global list. This may happen, if no master browser could be found
-              // during an IP scan. So, we will create a temporary workgroup item now.
-              Smb4KWorkgroup workgroup;
-              workgroup.setWorkgroupName(list.at(i)->workgroupName());
-
-              if (list.at(i)->isMasterBrowser())
-              {
-                workgroup.setMasterBrowserName(list.at(i)->hostName());
-                workgroup.setMasterBrowserIP(list.at(i)->ip());
-              }
-              else
-              {
-                // Do nothing
-              }
-
-              Smb4KNetworkBrowserItem *workgroup_item = new Smb4KNetworkBrowserItem(m_widget, &workgroup);
-              (void) new Smb4KNetworkBrowserItem(workgroup_item, list.at(i));
-              
-              if (Smb4KSettings::autoExpandNetworkItems() && !workgroup_item->isExpanded())
-              {
-                m_widget->expandItem(workgroup_item);
-              }
-              else
-              {
-                // Do nothing
-              }
-              continue;
-            }
-          }
-          else
-          {
-            // Do nothing
-          }
-        }
-        else
-        {
-          // This host is new. Add it to the list widget and create the
-          // workgroup item as well if needed.
-          QList<QTreeWidgetItem *> workgroups = m_widget->findItems(list.at(i)->workgroupName(), Qt::MatchFixedString, Smb4KNetworkBrowser::Network);
-          
-          if (!workgroups.isEmpty())
-          {
-            for (int j = 0; j < workgroups.size(); ++j)
-            {
-              Smb4KNetworkBrowserItem *workgroup_item = static_cast<Smb4KNetworkBrowserItem *>(workgroups.at(j));
-
-              if (workgroup_item)
-              {
-                // FIXME: Do we need to change the (pseudo) master browser here?
-                (void) new Smb4KNetworkBrowserItem(workgroup_item, list.at(i));
-                
-                if (Smb4KSettings::autoExpandNetworkItems() && !workgroup_item->isExpanded())
-                {
-                  m_widget->expandItem(workgroup_item);
-                }
-                else
-                {
-                  // Do nothing
-                }
-                continue;
-              }
-              else
-              {
-                continue;
-              }
-            }
-          }
-          else
-          {
-            // We need to create the workgroup and the host item. Since the workgroup
-            // was not found in the browser, we can assume, that it is also not available
-            // in the global list. This may happen, if no master browser could be found
-            // during an IP scan. So, we will create a temporary workgroup item now.
-            Smb4KWorkgroup workgroup;
-            workgroup.setWorkgroupName(list.at(i)->workgroupName());
-
-            if (list.at(i)->isMasterBrowser())
-            {
-              workgroup.setMasterBrowserName(list.at(i)->hostName());
-              workgroup.setMasterBrowserIP(list.at(i)->ip());
-            }
-            else
-            {
-              // Do nothing
-            }
-
-            Smb4KNetworkBrowserItem *workgroup_item = new Smb4KNetworkBrowserItem(m_widget, &workgroup);
-            (void) new Smb4KNetworkBrowserItem(workgroup_item, list.at(i));
-            
-            if (Smb4KSettings::autoExpandNetworkItems() && !workgroup_item->isExpanded())
-            {
-              m_widget->expandItem(workgroup_item);
-            }
-            else
-            {
-              // Do nothing
-            }
-            continue;
-          }
-        }
-      }
-      
-      // Sort the items.
-      m_widget->sortItems(Smb4KNetworkBrowser::Network, Qt::AscendingOrder);
-    }
-    else
-    {
-      // If the list of hosts is empty, we can clear the whole 
-      // network neighborhood tree.
-      m_widget->clear();
-    }
-  }
-}
-
-
-void Smb4KNetworkBrowserPart::slotShares(Smb4KHost *host, const QList<Smb4KShare *> &list)
-{
-  if (host)
-  {
-    QList<QTreeWidgetItem *> network_items = m_widget->findItems(host->hostName(), Qt::MatchFixedString|Qt::MatchRecursive, Smb4KNetworkBrowser::Network);
+    QMutableListIterator<QTreeWidgetItem *> it(workgroups);
     
-    // Find the host and process it.
-    for (int i = 0; i < network_items.size(); ++i)
+    while (it.hasNext())
     {
-      Smb4KNetworkBrowserItem *network_item = static_cast<Smb4KNetworkBrowserItem *>(network_items[i]);
+      QTreeWidgetItem *item = it.next();
       
-      if (network_item && network_item->type() == Host &&
-          QString::compare(network_item->hostItem()->workgroupName(), host->workgroupName(), Qt::CaseInsensitive) == 0)
+      if (item->type() == Workgroup)
       {
-        QStringList selected_items;
+        Smb4KNetworkBrowserItem *workgroupItem = static_cast<Smb4KNetworkBrowserItem *>(item);
+        QTreeWidgetItemIterator itemIt(workgroupItem);
         
-        // Delete all shares of the host.
-        while (network_item->childCount() != 0)
+        //
+        // Remove obsolete hosts and update existing ones
+        //
+        while (*itemIt)
         {
-          if (network_item->child(0)->isSelected())
+          Smb4KNetworkBrowserItem *networkItem = static_cast<Smb4KNetworkBrowserItem *>(*itemIt);
+          
+          if (networkItem->type() == Host)
           {
-            // Add item to the list of selected items.
-            selected_items << static_cast<Smb4KNetworkBrowserItem *>(network_item->child(0))->shareItem()->unc();
+            HostPtr host = findHost(networkItem->hostItem()->hostName(), networkItem->hostItem()->workgroupName());
+            
+            if (host)
+            {
+              networkItem->update();
+            }
+            else
+            {
+              delete networkItem;
+            }
           }
           else
           {
-            // Do nothing
-          }
-          delete network_item->child(0);
-        }
-        
-        // Add the newly discovered shares to the host.
-        if (!list.isEmpty())
-        {
-          // Expand the list of shares.
-          if (Smb4KSettings::autoExpandNetworkItems() && !network_item->isExpanded())
-          {
-            m_widget->expandItem(network_item);
-          }
-          else
-          {
-            // Do nothing
+            break;
           }
           
-          // Add the shares to the host.
-          for (int j = 0; j < list.size(); ++j)
+          ++itemIt;
+        }
+        
+        //
+        // Add new hosts to the workgroup item and remove obsolete workgroups if
+        // necessary. Honor the auto-expand feature.
+        //
+        QList<HostPtr> members = workgroupMembers(workgroupItem->workgroupItem());
+        
+        if (!members.isEmpty())
+        {
+          for (const HostPtr &host : members)
           {
-            Smb4KNetworkBrowserItem *item = new Smb4KNetworkBrowserItem(network_item, list.at(j));
-            item->setSelected(selected_items.contains(list.at(j)->unc()));
+            bool foundHost = false;
+            
+            for (int i = 0; i < workgroupItem->childCount(); ++i)
+            {
+              Smb4KNetworkBrowserItem *hostItem = static_cast<Smb4KNetworkBrowserItem *>(workgroupItem->child(i));
+              
+              if (hostItem->hostItem()->hostName() == host->hostName())
+              {
+                foundHost = true;
+                break;
+              }
+              else
+              {
+                continue;
+              }
+            }
+            
+            if (!foundHost)
+            {
+              (void) new Smb4KNetworkBrowserItem(workgroupItem, host);
+            }
+            else
+            {
+              // Do nothing
+            }
+          }
+          
+          // Auto-expand the workgroup item, if applicable
+          if (Smb4KSettings::autoExpandNetworkItems() && !workgroupItem->isExpanded())
+          {
+            m_widget->expandItem(workgroupItem);
+          }
+          else
+          {
+            // Do nothing
           }
         }
         else
         {
-          // Collapse the list of shares.
-          m_widget->collapseItem(network_item);
+          // Delete all hosts of the workgroup (if there should still be some) and
+          // remove the workgroup item from the view (no hosts => no workgroup)
+          while (workgroupItem->childCount() != 0)
+          {
+            delete workgroupItem->takeChild(0);
+          }
+          
+          delete workgroupItem;
         }
-
-        // Stop the loop.
-        break;
       }
       else
       {
@@ -1250,7 +883,9 @@ void Smb4KNetworkBrowserPart::slotShares(Smb4KHost *host, const QList<Smb4KShare
       }
     }
     
-    // Sort the items.
+    //
+    // Sort the items
+    //
     m_widget->sortItems(Smb4KNetworkBrowser::Network, Qt::AscendingOrder);
   }
   else
@@ -1260,66 +895,165 @@ void Smb4KNetworkBrowserPart::slotShares(Smb4KHost *host, const QList<Smb4KShare
 }
 
 
-void Smb4KNetworkBrowserPart::slotAddIPAddress(Smb4KHost *host)
+void Smb4KNetworkBrowserPart::slotShares(const HostPtr &host)
+{
+  if (host)
+  {
+    //
+    // Find the host(s)
+    //
+    QList<QTreeWidgetItem *> hosts = m_widget->findItems(host->hostName(), Qt::MatchFixedString|Qt::MatchRecursive, Smb4KNetworkBrowser::Network);
+    QMutableListIterator<QTreeWidgetItem *> it(hosts);
+    
+    while (it.hasNext())
+    {
+      Smb4KNetworkBrowserItem *hostItem = static_cast<Smb4KNetworkBrowserItem *>(it.next());
+      
+      if (hostItem->type() == Host && hostItem->hostItem()->workgroupName() == host->workgroupName())
+      {
+        QTreeWidgetItemIterator itemIt(hostItem);
+        
+        //
+        // Remove obsolete shares and update existing ones
+        //
+        while (*itemIt)
+        {
+          Smb4KNetworkBrowserItem *shareItem = static_cast<Smb4KNetworkBrowserItem *>(*itemIt);
+          
+          if (shareItem->type() == Share)
+          {
+            SharePtr share = findShare(shareItem->shareItem()->unc(), shareItem->shareItem()->workgroupName());
+            
+            if (share)
+            {
+              shareItem->update();
+            }
+            else
+            {
+              delete shareItem;
+            }
+          }
+          else
+          {
+            break;
+          }
+          
+          ++itemIt;
+        }
+        
+        //
+        // Add new shares to the host item. The host will not be removed from the
+        // view when it has no shares. Honor the auto-expand feature.
+        //
+        QList<SharePtr> shares = sharedResources(host);
+        
+        if (!shares.isEmpty())
+        {
+          for (const SharePtr &share : shares)
+          {
+            bool foundShare = false;
+            
+            for (int i = 0; i < hostItem->childCount(); ++i)
+            {
+              Smb4KNetworkBrowserItem *shareItem = static_cast<Smb4KNetworkBrowserItem *>(hostItem->child(i));
+              
+              if (shareItem->shareItem()->unc() == share->unc())
+              {
+                foundShare = true;
+                break;
+              }
+              else
+              {
+                continue;
+              }
+            }
+            
+            if (!foundShare)
+            {
+              (void) new Smb4KNetworkBrowserItem(hostItem, share);
+            }
+            else
+            {
+              // Do nothing
+            }
+          }
+          
+          // Auto-expand the host item, if applicable
+          if (Smb4KSettings::autoExpandNetworkItems() && !hostItem->isExpanded())
+          {
+            m_widget->expandItem(hostItem);
+          }
+          else
+          {
+            // Do nothing
+          }
+        }
+        else
+        {
+          // Delete all shares (if there should still be some), but leave the 
+          // host in the view.
+          while (hostItem->childCount() != 0)
+          {
+            delete hostItem->takeChild(0);
+          }
+        }
+      }
+      else
+      {
+        continue;
+      }
+    }
+    
+    //
+    // Sort the items
+    //
+    m_widget->sortItems(Smb4KNetworkBrowser::Network, Qt::AscendingOrder);
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
+void Smb4KNetworkBrowserPart::slotAddIPAddress(const HostPtr &host)
 {
   Q_ASSERT(host);
   
-  // Find the host.
-  Smb4KNetworkBrowserItem *hostItem = NULL;
-  QTreeWidgetItemIterator it(m_widget);
-  
-  while(*it)
+  //
+  // Process the IP address
+  //
+  if (host)
   {
-    Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>(*it);
+    //
+    // Find the host and update it. Also set the IP address of the master 
+    // browser
+    //
+    QTreeWidgetItemIterator itemIt(m_widget);
     
-    if (item)
+    while (*itemIt)
     {
-      if (item->type() == Host)
-      {
-        if (QString::compare(host->unc(), item->hostItem()->unc(), Qt::CaseInsensitive) == 0 &&
-             QString::compare(host->workgroupName(), item->hostItem()->workgroupName(), Qt::CaseInsensitive) == 0)
-        {
-          hostItem = item;
-          break;
-        }
-        else
-        {
-          // Do nothing
-        }
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-    else
-    {
-      // Do nothing
-    }
-    
-    ++it;
-  }
-  
-  if (hostItem)
-  {
-    // Update host item
-    hostItem->update(host);
-    
-    // If the host is a master browser, set the IP address of the
-    // workgroup item.
-    Smb4KNetworkBrowserItem *workgroupItem = NULL;
-    
-    if (host->isMasterBrowser())
-    {
-      workgroupItem = static_cast<Smb4KNetworkBrowserItem *>(hostItem->parent());
+      Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>(*itemIt);
       
-      if (workgroupItem)
+      if (item->type() == Host && item->hostItem()->hostName() == host->hostName() 
+          && item->hostItem()->workgroupName() == host->workgroupName())
       {
-        Smb4KWorkgroup *workgroup = findWorkgroup(host->workgroupName());
+        // Update the host
+        item->update();
         
-        if (workgroup)
+        // Update the shares
+        for (int i = 0; i < item->childCount(); ++i)
         {
-          workgroupItem->update(workgroup);
+          Smb4KNetworkBrowserItem *shareItem = static_cast<Smb4KNetworkBrowserItem *>(item->child(i));
+          shareItem->update();
+        }
+        
+        // Update the workgroup master browser's IP address
+        Smb4KNetworkBrowserItem *workgroupItem = static_cast<Smb4KNetworkBrowserItem *>(item->parent());
+        
+        if (workgroupItem && host->isMasterBrowser())
+        {
+          workgroupItem->update();
         }
         else
         {
@@ -1330,41 +1064,8 @@ void Smb4KNetworkBrowserPart::slotAddIPAddress(Smb4KHost *host)
       {
         // Do nothing
       }
-    }
-    else
-    {
-      // Do nothing
-    }
-    
-    // If there are already shared resources added to the
-    // host item, update their IP address entry.
-    if (hostItem->childCount() != 0)
-    {
-      for (int i = 0; i < hostItem->childCount(); ++i)
-      {
-        Smb4KNetworkBrowserItem *shareItem = static_cast<Smb4KNetworkBrowserItem *>(hostItem->child(i));
-        
-        if (shareItem)
-        {
-          // We only need to update the IP address. No need to
-          // use Smb4KNetworkBrowserItem::update() here.
-          shareItem->shareItem()->setHostIP(host->ip());
-        }
-        else
-        {
-          // Do nothing
-        }
-      }
-    }
-    
-    // Now adjust the IP address column, if it is not hidden.
-    if (!m_widget->isColumnHidden(Smb4KNetworkBrowser::IP))
-    {
-      m_widget->resizeColumnToContents(Smb4KNetworkBrowser::IP);
-    }
-    else
-    {
-      // Do nothing
+      
+      ++itemIt;
     }
   }
   else
@@ -1374,7 +1075,7 @@ void Smb4KNetworkBrowserPart::slotAddIPAddress(Smb4KHost *host)
 }
 
 
-void Smb4KNetworkBrowserPart::slotAuthError(Smb4KHost *host, int process)
+void Smb4KNetworkBrowserPart::slotAuthError(const HostPtr &host, int process)
 {
   switch (process)
   {
@@ -1597,7 +1298,7 @@ void Smb4KNetworkBrowserPart::slotCustomOptions(bool /*checked*/)
 void Smb4KNetworkBrowserPart::slotAddBookmark(bool /*checked*/)
 {
   QList<QTreeWidgetItem *> items = m_widget->selectedItems();
-  QList<Smb4KShare *> shares;
+  QList<SharePtr> shares;
 
   if (!items.isEmpty())
   {
@@ -1689,7 +1390,7 @@ void Smb4KNetworkBrowserPart::slotMountActionTriggered(bool /*checked*/)
     // mounted shares, we use the number of unmounted shares. If that is
     // greater than 0, we mount all shares that need to be mounted, otherwise
     // we unmount all selected shares.
-    QList<Smb4KShare *> unmounted, mounted;
+    QList<SharePtr> unmounted, mounted;
     
     for (int i = 0; i < items.size(); ++i)
     {
@@ -1772,7 +1473,7 @@ void Smb4KNetworkBrowserPart::slotMountActionChanged(bool active)
 }
 
 
-void Smb4KNetworkBrowserPart::slotScannerAboutToStart(Smb4KBasicNetworkItem *item, int process)
+void Smb4KNetworkBrowserPart::slotScannerAboutToStart(const NetworkItemPtr &item, int process)
 {
   switch (process)
   {
@@ -1792,7 +1493,7 @@ void Smb4KNetworkBrowserPart::slotScannerAboutToStart(Smb4KBasicNetworkItem *ite
     {
       if (!m_silent)
       {
-        Smb4KWorkgroup *workgroup = static_cast<Smb4KWorkgroup *>(item);
+        WorkgroupPtr workgroup = item.staticCast<Smb4KWorkgroup>();
         emit setStatusBarText(i18n("Looking for hosts in domain %1...", workgroup->workgroupName()));
       }
       else
@@ -1805,7 +1506,7 @@ void Smb4KNetworkBrowserPart::slotScannerAboutToStart(Smb4KBasicNetworkItem *ite
     {
       if (!m_silent)
       {
-        Smb4KHost *host = static_cast<Smb4KHost *>(item);
+        HostPtr host = item.staticCast<Smb4KHost>();
         emit setStatusBarText(i18n("Looking for shares provided by host %1...", host->hostName()));
       }
       else
@@ -1860,7 +1561,7 @@ void Smb4KNetworkBrowserPart::slotScannerAboutToStart(Smb4KBasicNetworkItem *ite
 }
 
 
-void Smb4KNetworkBrowserPart::slotScannerFinished(Smb4KBasicNetworkItem */*item*/, int /*process*/)
+void Smb4KNetworkBrowserPart::slotScannerFinished(const NetworkItemPtr &/*item*/, int /*process*/)
 {
   if (!m_silent)
   {
@@ -1947,7 +1648,7 @@ void Smb4KNetworkBrowserPart::slotMounterFinished(int process)
 }
 
 
-void Smb4KNetworkBrowserPart::slotShareMounted(Smb4KShare *share)
+void Smb4KNetworkBrowserPart::slotShareMounted(const SharePtr &share)
 {
   QTreeWidgetItemIterator it(m_widget);
   
@@ -1959,7 +1660,7 @@ void Smb4KNetworkBrowserPart::slotShareMounted(Smb4KShare *share)
     {
       if (QString::compare(item->shareItem()->unc(), share->unc(), Qt::CaseInsensitive) == 0)
       {
-        item->update(share);
+        item->update();
         break;
       }
       else
@@ -1977,7 +1678,7 @@ void Smb4KNetworkBrowserPart::slotShareMounted(Smb4KShare *share)
 }
 
 
-void Smb4KNetworkBrowserPart::slotShareUnmounted(Smb4KShare *share)
+void Smb4KNetworkBrowserPart::slotShareUnmounted(const SharePtr &share)
 {
   QTreeWidgetItemIterator it(m_widget);
   
@@ -1989,7 +1690,7 @@ void Smb4KNetworkBrowserPart::slotShareUnmounted(Smb4KShare *share)
     {
       if (QString::compare(item->shareItem()->unc(), share->unc(), Qt::CaseInsensitive) == 0)
       {
-        item->update(share);
+        item->update();
         break;
       }
       else
