@@ -2,7 +2,7 @@
     smb4kprofilesmenu  -  The menu for the profiles
                              -------------------
     begin                : Do Aug 10 2014
-    copyright            : (C) 2014-2015 by Alexander Reinholdt
+    copyright            : (C) 2014-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -44,22 +44,8 @@ Smb4KProfilesMenu::Smb4KProfilesMenu(QObject* parent)
 : KSelectAction(KDE::icon("format-list-unordered"), i18n("Profiles"), parent)
 {
   QStringList profiles = Smb4KProfileManager::self()->profilesList();
+  slotProfilesListChanged(profiles);
   
-  for (int i = 0; i < profiles.size(); ++i)
-  {
-    QAction *action = addAction(profiles.at(i));
-    
-    if (action)
-    {
-      action->setEnabled(Smb4KProfileManager::self()->useProfiles());
-    }
-    else
-    {
-      // Do nothing
-    }
-  }
-  
-  setCurrentAction(Smb4KProfileManager::self()->activeProfile());
   setToolBarMode(KSelectAction::MenuMode);
   
   // Connections
@@ -67,6 +53,8 @@ Smb4KProfilesMenu::Smb4KProfilesMenu(QObject* parent)
           this, SLOT(slotActiveProfileChanged(QString)));
   connect(Smb4KProfileManager::self(), SIGNAL(profilesListChanged(QStringList)),
           this, SLOT(slotProfilesListChanged(QStringList)));
+  connect(Smb4KProfileManager::self(), SIGNAL(profileUsageChanged(bool)),
+          this, SLOT(slotProfileUsageChanged(bool)));
   connect(this, SIGNAL(triggered(QString)), 
           this, SLOT(slotActionTriggered(QString)));
 }
@@ -77,20 +65,21 @@ Smb4KProfilesMenu::~Smb4KProfilesMenu()
 }
 
 
-
-void Smb4KProfilesMenu::slotActiveProfileChanged(const QString& newProfile)
-{
-  setCurrentAction(newProfile);
-}
-
-
-void Smb4KProfilesMenu::slotProfilesListChanged(const QStringList& profiles)
+void Smb4KProfilesMenu::refreshMenu()
 {
+  //
+  // Clear the select action
+  //
   clear();
+
+  //
+  // Get the list of profiles and add all profiles to the select action
+  //
+  QStringList profiles = Smb4KProfileManager::self()->profilesList();
   
-  for (int i = 0; i < profiles.size(); ++i)
+  for (const QString &profile : profiles)
   {
-    QAction *action = addAction(profiles.at(i));
+    QAction *action = addAction(profile);
     
     if (action)
     {
@@ -102,16 +91,35 @@ void Smb4KProfilesMenu::slotProfilesListChanged(const QStringList& profiles)
     }
   }
   
+  //
+  // Enable the action if the user chose to use profiles
+  //
+  setEnabled(Smb4KProfileManager::self()->useProfiles());
+  
+  //
+  // Set the current action
+  // 
   setCurrentAction(Smb4KProfileManager::self()->activeProfile());
 }
 
 
+
+void Smb4KProfilesMenu::slotActiveProfileChanged(const QString& newProfile)
+{
+  setCurrentAction(newProfile);
+}
+
+
+void Smb4KProfilesMenu::slotProfilesListChanged(const QStringList& /*profiles*/)
+{
+  refreshMenu();
+}
+
+
 void Smb4KProfilesMenu::slotProfileUsageChanged(bool use)
 {
-  for (int i = 0; i < actions().size(); ++i)
+  for (QAction *action : actions())
   {
-    QAction *action = actions()[i];
-    
     if (action)
     {
       action->setEnabled(use);
@@ -121,6 +129,8 @@ void Smb4KProfilesMenu::slotProfileUsageChanged(bool use)
       // Do nothing
     }
   }
+  
+  setEnabled(use);
 }
 
 
