@@ -2,7 +2,7 @@
     smb4ksharesmenu  -  Shares menu
                              -------------------
     begin                : Mon Sep 05 2011
-    copyright            : (C) 2011-2016 by Alexander Reinholdt
+    copyright            : (C) 2011-2018 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -26,6 +26,9 @@
 #ifndef SMB4KSHARESMENU_H
 #define SMB4KSHARESMENU_H
 
+// application specific includes
+#include "core/smb4kglobal.h"
+
 // Qt includes
 #include <QAction>
 #include <QActionGroup>
@@ -45,8 +48,7 @@ class Smb4KSharesMenu : public KActionMenu
     /**
      * Constructor
      */
-    explicit Smb4KSharesMenu(QWidget *parentWidget = 0,
-                              QObject *parent = 0);
+    explicit Smb4KSharesMenu(QWidget *parentWidget = 0, QObject *parent = 0);
 
     /**
      * Destructor
@@ -60,20 +62,10 @@ class Smb4KSharesMenu : public KActionMenu
 
   protected slots:
     /**
-     * Connected to Smb4KMounter::mounted() signal. Adds a
-     * share to the menu.
-     *
-     * @param share         The share that has been mounted
+     * This slot is connected to the Smb4KMounter::mountedSharesListChanged()
+     * signal. It refreshes the menu.
      */
-    void slotShareMounted(Smb4KShare *share);
-
-    /**
-     * Connected to Smb4KMounter::unmounted() signal. Removes
-     * a share from the menu.
-     *
-     * @param share         The share that has been unmounted
-     */
-    void slotShareUnmounted(Smb4KShare *share);
+    void slotMountedSharesListChanged();
 
     /**
      * This slot unmounts all shares at once.
@@ -98,11 +90,6 @@ class Smb4KSharesMenu : public KActionMenu
      */
     QWidget *m_parent_widget;
 
-    /**
-     * The action collection
-     */
-    KActionCollection *m_action_collection;
-
     /**
      * Share menus
      */
