@@ -2,7 +2,7 @@
     smb4ksystemtray  -  This is the system tray window class of Smb4K.
                              -------------------
     begin                : Mi Jun 13 2007
-    copyright            : (C) 2007-2016 by Alexander Reinholdt
+    copyright            : (C) 2007-2017 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -40,6 +40,7 @@
 
 // Qt includes
 #include <QMenu>
+#include <QDebug>
 
 // KDE specific includes
 #include <KIconThemes/KIconLoader>
@@ -49,6 +50,7 @@
 #include <KXmlGui/KActionCollection>
 #include <KCoreAddons/KPluginLoader>
 #include <KCoreAddons/KPluginFactory>
+#include <KCoreAddons/KAboutData>
 #include <KWidgetsAddons/KMessageBox>
 
 using namespace Smb4KGlobal;
@@ -57,48 +59,55 @@ using namespace Smb4KGlobal;
 Smb4KSystemTray::Smb4KSystemTray(QWidget *parent)
 : KStatusNotifierItem("smb4k_systemtray", parent)
 {
-  setIconByName("smb4k");
-  setToolTip(KDE::icon("smb4k"), i18n("Smb4K"), i18n("Advanced Network Neighborhood Browser"));
+  //
+  // Set the icon for the system tray
+  //
+  QString iconName = QStringLiteral("network-workgroup-symbolic");
+  setIconByName(iconName);
   
-  // Show the icon to the user. It will become passive, if the scanner
-  // could not find something and no shares were mounted.
-  setStatus(KStatusNotifierItem::Active);
-
-  // Set up the context menu (skeleton):
-  QAction *manual_mount = new QAction(KDE::icon("view-form", QStringList("emblem-mounted")),
-                          i18n("&Open Mount Dialog"), this);
-  QAction *configure = KStandardAction::preferences(this, SLOT(slotConfigDialog()),
-                       this);
+  //
+  // Set the tooltip text
+  //
+  setToolTip(iconName, i18n("Smb4K"), KAboutData::applicationData().shortDescription());
   
-  Smb4KSharesMenu *shares_menu = new Smb4KSharesMenu(associatedWidget(), this);
-  Smb4KBookmarkMenu *bookmark_menu = new Smb4KBookmarkMenu(Smb4KBookmarkMenu::SystemTray, associatedWidget(), this);
-  Smb4KProfilesMenu *profiles_menu = new Smb4KProfilesMenu(this);
+  //
+  // Set the status of the icon. By default, it is active. It will become passive, 
+  // if the scanner could not find something and no shares were mounted.
+  //
+  setStatus(Active);
   
-  addAction("manual_mount", manual_mount);
-  addAction("configure", configure);
-  addAction("shares_menu", shares_menu);
-  addAction("bookmark_menu", bookmark_menu);
-  addAction("profiles_menu", profiles_menu);
-
-  contextMenu()->addAction(shares_menu);
-  contextMenu()->addAction(bookmark_menu);
-  contextMenu()->addAction(profiles_menu);
+  //
+  // Set the category
+  //
+  setCategory(ApplicationStatus);
+  
+  //
+  // Add the actions to the action collection
+  //
+  QAction *mountAction = new QAction(KDE::icon("view-form", QStringList("emblem-mounted")), i18n("&Open Mount Dialog"), this);
+  connect(mountAction, SIGNAL(triggered(bool)), SLOT(slotMountDialog()));
+    
+  addAction("shares_menu", new Smb4KSharesMenu(associatedWidget()));
+  addAction("bookmarks_menu", new Smb4KBookmarkMenu(Smb4KBookmarkMenu::SystemTray, associatedWidget()));
+  addAction("profiles_menu", new Smb4KProfilesMenu());
+  addAction("mount_action", mountAction);
+  addAction("config_action", KStandardAction::preferences(this, SLOT(slotConfigDialog()), this));
+  
+  //
+  // Set up the menu
+  //  
+  contextMenu()->addAction(action("shares_menu"));
+  contextMenu()->addAction(action("bookmarks_menu"));
+  contextMenu()->addAction(action("profiles_menu"));
   contextMenu()->addSeparator();
-  contextMenu()->addAction(manual_mount);
-  contextMenu()->addAction(configure);
-
+  contextMenu()->addAction(action("mount_action"));
+  contextMenu()->addAction(action("config_action"));
+  
+  // 
   // Connections:
-  connect(manual_mount,                 SIGNAL(triggered(bool)),
-           this,                         SLOT(slotMountDialog(bool)));
-
-  connect(Smb4KMounter::self(),         SIGNAL(mounted(Smb4KShare*)),
-           this,                         SLOT(slotSetStatus()));
-
-  connect(Smb4KMounter::self(),         SIGNAL(unmounted(Smb4KShare*)),
-           this,                         SLOT(slotSetStatus()));
-
-  connect(Smb4KScanner::self(),         SIGNAL(workgroups(QList<Smb4KWorkgroup*>)),
-           this,                         SLOT(slotSetStatus()));
+  // 
+  connect(Smb4KMounter::self(), SIGNAL(mountedSharesListChanged()), SLOT(slotSetStatus()));
+  connect(Smb4KScanner::self(), SIGNAL(workgroups()), SLOT(slotSetStatus()));
 }
 
 
@@ -109,24 +118,42 @@ Smb4KSystemTray::~Smb4KSystemTray()
 
 void Smb4KSystemTray::loadSettings()
 {
-  // Adjust the bookmarks menu.
-  Smb4KBookmarkMenu *bookmark_menu = findChild<Smb4KBookmarkMenu *>();
+  //
+  // Adjust the bookmarks menu
+  //
+  Smb4KBookmarkMenu *bookmarkMenu = static_cast<Smb4KBookmarkMenu *>(action("bookmarks_menu"));
 
-  if (bookmark_menu)
+  if (bookmarkMenu)
   {
-    bookmark_menu->refreshMenu();
+    bookmarkMenu->refreshMenu();
   }
   else
   {
     // Do nothing
   }
 
-  // Adjust the shares menu.
-  Smb4KSharesMenu *shares_menu = findChild<Smb4KSharesMenu *>();
+  // 
+  // Adjust the shares menu
+  //
+  Smb4KSharesMenu *sharesMenu = static_cast<Smb4KSharesMenu *>(action("shares_menu"));
 
-  if (shares_menu)
+  if (sharesMenu)
   {
-    shares_menu->refreshMenu();
+    sharesMenu->refreshMenu();
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  //
+  // Adjust the profiles menu
+  //
+  Smb4KProfilesMenu *profilesMenu = static_cast<Smb4KProfilesMenu *>(action("profiles_menu"));
+  
+  if (profilesMenu)
+  {
+    profilesMenu->refreshMenu();
   }
   else
   {
@@ -139,7 +166,7 @@ void Smb4KSystemTray::loadSettings()
 // SLOT IMPLEMENTATIONS
 /////////////////////////////////////////////////////////////////////////////
 
-void Smb4KSystemTray::slotMountDialog(bool /* checked */)
+void Smb4KSystemTray::slotMountDialog()
 {
   Smb4KMounter::self()->openMountDialog(associatedWidget());
 }
@@ -147,54 +174,45 @@ void Smb4KSystemTray::slotMountDialog(bool /* checked */)
 
 void Smb4KSystemTray::slotConfigDialog()
 {
-  // If the config dialog is already created and cached,
-  // we do not create a new one but show the old instead:
-  KConfigDialog *dlg = NULL;
-
-  if ((dlg = KConfigDialog::exists("ConfigDialog")) && KConfigDialog::showDialog("ConfigDialog"))
+  //
+  // Check if the configuration dialog exists and try to show it.
+  //
+  if (KConfigDialog::exists("ConfigDialog"))
   {
-    // To make sure we do not connect the config dialog several times
-    // to slotSettingsChanged(), we break the connection first and re-
-    // establish it afterwards:
-    disconnect(dlg,  SIGNAL(settingsChanged(QString)),
-                this, SLOT(slotSettingsChanged(QString)));
-
-    connect(dlg,  SIGNAL(settingsChanged(QString)),
-             this, SLOT(slotSettingsChanged(QString)));
-
+    KConfigDialog::showDialog("ConfigDialog");
     return;
   }
   else
   {
     // Do nothing
   }
-
-  // Load the configuration dialog:
+  
+  //
+  // If the dialog does not exist, load and show it:
+  //
   KPluginLoader loader("smb4kconfigdialog");
-  KPluginFactory *config_factory = loader.factory();
+  KPluginFactory *configFactory = loader.factory();
 
-  if (config_factory)
+  if (configFactory)
   {
+    KConfigDialog *dlg = 0;
+    
     if (associatedWidget())
     {
-      dlg = config_factory->create<KConfigDialog>(associatedWidget());
+      dlg = configFactory->create<KConfigDialog>(associatedWidget());
       dlg->setObjectName("ConfigDialog");
     }
     else
     {
-      dlg = config_factory->create<KConfigDialog>(contextMenu());
+      dlg = configFactory->create<KConfigDialog>(contextMenu());
       dlg->setObjectName("ConfigDialog");
     }
-
-    // ... and show it.
+    
     if (dlg)
     {
-      connect(dlg,  SIGNAL(settingsChanged(QString)),
-               this, SLOT(slotSettingsChanged(QString)));
-
-      connect(dlg,  SIGNAL(settingsChanged(QString)),
-               this, SIGNAL(settingsChanged(QString)));
-
+      dlg->setObjectName("ConfigDialog");
+      connect(dlg, SIGNAL(settingsChanged(QString)), this, SLOT(slotSettingsChanged(QString)), Qt::UniqueConnection);
+      connect(dlg, SIGNAL(settingsChanged(QString)), this, SIGNAL(settingsChanged(QString)), Qt::UniqueConnection);
       dlg->show();
     }
     else
@@ -211,14 +229,18 @@ void Smb4KSystemTray::slotConfigDialog()
 
 void Smb4KSystemTray::slotSettingsChanged(const QString &)
 {
-  // Execute loadSettings():
+  // 
+  // Execute loadSettings()
+  // 
   loadSettings();
 }
 
 
 void Smb4KSystemTray::slotSetStatus()
 {
-  // Set the status of the system tray icon.
+  //
+  // Set the status of the system tray icon
+  // 
   if (!mountedSharesList().isEmpty() || !workgroupsList().isEmpty())
   {
     setStatus(KStatusNotifierItem::Active);
@@ -229,3 +251,4 @@ void Smb4KSystemTray::slotSetStatus()
   }
 }
 
+
